/* Globals */
struct tcp_pkt_array *tcp_pkts;
time_t start_receive_time;

/* Pcap handler */
pcap_t *recv_d;
pcap_dumper_t *dump;

/* Trace file handler */
#ifdef HAVE_APSRLIB_LIBZ_SUPPORT
gzFile file_handle;
#else
FILE *file_handle;
#endif

/* Link layer sizes */
unsigned int data_link_size, link_layer;

/* Other stuff */
char count_bytes, verbose, calc_checksums, statistic, beep_on, aprecv_daemon;
unsigned short index_number, aprecv_id; /* TODO move in struct */

/* Expand options */
unsigned char expand_ethernet, expand_arp, expand_pppoe, expand_token, expand_llc, expand_fddi, expand_all, no_dissection;

/* apsrlib(sniffing) support */
unsigned char use_apsrlib;

#ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
/* libbluetooth with apsrlib functions support */
unsigned char use_apsrlib_bluetooth;
/* File descriptor for socket operations */
int apsrlib_bluetooth_fd;
#endif

/* Module support */
#ifdef HAVE_APSRLIB_MODULE_SUPPORT
char module_ignore, modules_in_path;    /* Module options */
char *module_name;		
#define MAX_PATH_LENGTH		1024	/* Maximum MAX_PATH */
unsigned short module_length;
#endif

/* Default APSR Server Port */
#define APSRSERVER_PORT			2777
/* Max Buffer length of packets */
#define APSRSERVER_BUFFER_LENGTH	65535

/* aprecv socket */
int sock_aprecv_clt;

/* Hex printing */
int print_raw_hex, print_payload_ip_hex, print_payload_pppoe_hex, print_payload_udp_hex, print_payload_tcp_hex, print_tcp_stream;

/* Text printing */
int print_payload_tcp_text, print_payload_udp_text;

/* Logfiles, times, device and server addr */
char *aprecv_log_raw_file, *aprecv_log_file, *start_time, *device, *pcap_log_file, *iptrace_trace_file, *snoop_trace_file, *server_addr;

/* Statistics */
unsigned long all_counted_bytes;

struct statistic {
  struct count {
    unsigned long ethernetII;
    unsigned long ethernet8023raw;
    unsigned long ethernet8023;
    unsigned long ethernet8023snap;
    unsigned long vlan;
    unsigned long ipv4;
    unsigned long ipv6;
    unsigned long ipv6_rt;
    unsigned long ipv6_hh;
    unsigned long ipv7;
    unsigned long pppoe;
    unsigned long pppoe_tag;
    unsigned long ppp;
    unsigned long ppp_pap;
    unsigned long ppp_chap;
    unsigned long ppp_ipcp;
    unsigned long ppp_ipv6cp;
    unsigned long ppp_ipxcp;
    unsigned long ppp_atcp;
    unsigned long ppp_lcp;
    unsigned long ppp_ecp;
    unsigned long ppp_ccp;
    unsigned long ppp_eap;
    unsigned long ppp_bcp;
    unsigned long ppp_lqr;
    unsigned long ppp_decnetcp;
    unsigned long ppp_osinlcp;
    unsigned long ppp_snacp;
    unsigned long arp;
    unsigned long tcp;
    unsigned long udp;
    unsigned long icmpv4;
    unsigned long icmpv6;
    unsigned long igmpv0;
    unsigned long igmpv1;
    unsigned long igmpv2;
    unsigned long rgmp;
    unsigned long ripv1;
    unsigned long ripv2;
    unsigned long ipx;
    unsigned long spx;
    unsigned long spx2;
    unsigned long ipah;
    unsigned long ospfv2;
    unsigned long ospfv3;
    unsigned long egp;
    unsigned long bgp;
    unsigned long ipcomp;
    unsigned long narp;
    unsigned long ggp;
    unsigned long igrp;
    unsigned long eigrp;
    unsigned long ipxrip;
    unsigned long sctp;
    unsigned long pimv1;
    unsigned long pimv2;
    unsigned long ipesp;
    unsigned long ieee80211;
    unsigned long ripng;
    unsigned long grev0;
    unsigned long grev1;
    unsigned long ifmp;
    unsigned long prism_mon;
    unsigned long cdp;
    unsigned long slip;
    unsigned long token_ring;
    unsigned long fddi;
    unsigned long vrrp;
    unsigned long irtp;
    unsigned long mopv3_dumpload;
    unsigned long mpls;
    unsigned long cbt;
    unsigned long rsvp;
    unsigned long atm_rfc1483;
    unsigned long atm_clip_llc;
    unsigned long atm_clip_withoutllc;
    unsigned long sunatm;
    unsigned long chdlc;
    unsigned long msdp;
    unsigned long cgmp;
    unsigned long bofl;
    unsigned long gsmp;
    unsigned long vtp;
    unsigned long etherip;
    unsigned long mtp;
    unsigned long pgm;
    unsigned long rdpv1;
    unsigned long rdpv2;
    unsigned long netblk;
    unsigned long tmux_mini;
    unsigned long luni;
    unsigned long sdrp;
    unsigned long all;
  } c;
  struct error {
    unsigned long ipv4;
    unsigned long ppp_ipcp;
    unsigned long tcp;
    unsigned long arp;
  } err;
  struct checksum_error {
    unsigned long ipv4;
    unsigned long tcp;
    unsigned long udp;
    unsigned long icmpv4;
    unsigned long igmpv0;
    unsigned long igmpv1;
    unsigned long igmpv2;
    unsigned long rgmp;
    unsigned long igrp;
    unsigned long eigrp;
    unsigned long egp;
    unsigned long pimv1;
    unsigned long pimv2;
    unsigned long irtp;
  } cr;
  struct trunc {
    unsigned long ethernetII;
    unsigned long ieee80211;
    unsigned long cdp;
    unsigned long token_ring;
    unsigned long fddi;
  } tr;
} stats;

