#include "../include/aprecv-general.h"

int open_server_connection (char *server_addr, unsigned short server_port) {
        static struct sockaddr_in servAddr;
	static struct hostent *hp;
        static struct apsrserver_protocol_header *recv_header, send_header;
        static char aprecv_buf[APSRSERVER_BUFFER_LENGTH];
	
        apsr_output("Open connection to APSR-Server(%s:%d).\n", 
			server_addr, server_port);
	      
        if((sock_aprecv_clt = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                apsr_stream_output((FILE *) stderr, "Error: Can't open socket.\n");
                #ifdef APSRSERVER_DEBUG
                apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
                                strerror(errno));
                #endif
                exit(EXIT_FAILURE);
        }

	memset(&servAddr, 0, sizeof(servAddr));
        servAddr.sin_family = AF_INET;
	
        if((hp=gethostbyname(server_addr)) == NULL) {
		#ifdef APRECV_DEBUG
	        herror("Error occured while gethostbyname(server_addr).\n");
		#endif
        }
	
        memcpy(&servAddr.sin_addr, hp->h_addr,(size_t) hp->h_length);
        servAddr.sin_port = (in_port_t) apsr_htons(server_port);

        if(connect(sock_aprecv_clt,(struct sockaddr *) &servAddr,(socklen_t) sizeof(servAddr)) < 0) {
                apsr_stream_output((FILE *) stderr, "Error: Can't connect to server.\n");
                exit(EXIT_FAILURE);
        }

        send_header.version = APSR_APSRSERVER_VERSION;
        send_header.id      = apsr_htons(0);       /* TODO */
        send_header.command = APSR_APSRSERVER_REQ_LOGIN;
        send_header.code    = APSR_APSRSERVER_REQ_LOGIN_COD_LOGIN;
        send_header.length  = apsr_htons((unsigned short) sizeof(struct apsrserver_protocol_header));

	if(send(sock_aprecv_clt, (unsigned char *)&send_header, 
				sizeof(send_header), 0) < 0) {
                #ifdef APRECV_DEBUG
                apsr_stream_output((FILE *) stderr, "Error: Sending(login) failure.\n");
                #endif
                exit(EXIT_FAILURE);
        }
	
	while(1) {
                memset(&aprecv_buf, 0, APSRSERVER_BUFFER_LENGTH); 
		if((recv(sock_aprecv_clt, aprecv_buf, APSRSERVER_BUFFER_LENGTH, 0)) < 0) { 
			#ifdef APRECV_DEBUG
	                apsr_stream_output((FILE *) stderr, "Error: Receive from apsrserver failed.\n");
                        #endif
                        exit(EXIT_FAILURE);
                }
		
                recv_header = (struct apsrserver_protocol_header *)aprecv_buf;

		if(recv_header->version != APSR_APSRSERVER_VERSION) {
		          #ifdef APRECV_DEBUG
                	  apsr_output("Error: Incompatibel packet version received from apsrserver, ignore.\n");
			  #endif
		}	  
                else {
 	                  switch(recv_header->command) {
	                    case APSR_APSRSERVER_REP_LOGIN:
	                      switch(recv_header->code) {
	                        case APSR_APSRSERVER_REP_LOGIN_COD_OK:
			          aprecv_id = apsr_ntohs(recv_header->id);
				  goto got_id;
				  /* Never reached */
			          break;
			        default:
			          break;
			      }
			      break;
		            default:
			      break;
		      	  }    
		}
	}
	
got_id: 
		
	/* TODO server_auth(); */
	
	apsr_output("Connected.\n");
	
	return(RET_SUCCESS);
}

int close_server_connection(void) {
        static struct apsrserver_protocol_header end_header;

	end_header.version = APSR_APSRSERVER_VERSION;
	end_header.id      = apsr_htons(aprecv_id);
	end_header.command = APSR_APSRSERVER_REQ_LOGIN;
	end_header.code	   = APSR_APSRSERVER_REQ_LOGIN_COD_LOGOUT;
	end_header.length  = apsr_htons((unsigned short) sizeof(struct apsrserver_protocol_header));

        if(send(sock_aprecv_clt, (unsigned char *)&end_header, 
				sizeof(end_header), 0) < 0) {
		#ifdef APRECV_DEBUG
		apsr_stream_output((FILE *) stderr, "Error: Sending(close_server_connection) failure.\n");
		#endif
                exit(EXIT_FAILURE);
        }

	if(close(sock_aprecv_clt) < 0) {
		#ifdef APRECV_DEBUG
                apsr_stream_output((FILE *) stderr, "Error: close(sock_aprecv_clt) failed.\n");
	        #endif
        }

	return(RET_SUCCESS);
}

int send_server_got_packet(const unsigned char *packet, unsigned short length) {
	static struct apsrserver_protocol_header send_header;
	unsigned char *buffer;

	buffer = (unsigned char *) xmalloc(sizeof(struct apsrserver_protocol_header)+length+1);
	
        send_header.version = APSR_APSRSERVER_VERSION;
	send_header.id      = apsr_htons(aprecv_id);
	send_header.command = APSR_APSRSERVER_GOT_PACKET;
	send_header.code    = APSR_APSRSERVER_GOT_PACKET_COD_PLAIN;
	send_header.length  = apsr_htons(sizeof(struct apsrserver_protocol_header)+length);
	
        memcpy(buffer + sizeof(struct apsrserver_protocol_header), (unsigned char *)&packet, (size_t) length);
        memcpy((unsigned char *)buffer, (unsigned char *)&send_header, sizeof(struct apsrserver_protocol_header));

        if(send(sock_aprecv_clt, buffer,
		sizeof(struct apsrserver_protocol_header)+length, 0) < 0) {
		#ifdef APRECV_DEBUG
		apsr_stream_output((FILE *) stderr, "Error: Sending(send_server_got_packet) failure.\n");
		#endif
                exit(EXIT_FAILURE);
        }

	free(buffer);

        return(RET_SUCCESS);
}			
