#include "../include/aprecv-general.h"

static const char *ifmp_opcodes[]= {
	" (Syn)",
	" (Syn/Ack)",
	" (Rst/Ack)",
	" (Ack)",
	" (Redirect)",
	" (Reclaim)",
	" (Reclaim/Ack)",
	" (Label Range)",
	" (Error)",
};

/* Adjacency & Redirect Protocol */
int print_ifmp_packet (unsigned char *packet, int opt_size) {
  static struct ifmp_pkt_header *ifmp_header;

  ifmp_header 	= (struct ifmp_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("- IFMP Header -\n  Version: %d\n", ifmp_header->version);
  apsr_output("  Opcode: %d", ifmp_header->opcode);
  if(verbose) 
  	apsr_output("%s", ifmp_opcodes[ifmp_header->opcode]);
  apsr_output(", Checksum: 0x%04x\n", ifmp_header->sum);

  switch(ifmp_header->opcode) {
	case APSR_IFMP_OPCODE_SYN:
	case APSR_IFMP_OPCODE_SYNACK:
 	case APSR_IFMP_OPCODE_RSTACK:
	case APSR_IFMP_OPCODE_ACK:  
          apsr_output("  Sender Instance: %u, Peer Instance: %u\n", 
		  apsr_ntohl(ifmp_header->ext.adjacency.sender_inst),
		  apsr_ntohl(ifmp_header->ext.adjacency.peer_inst));
          apsr_output("  Peer Identity: %u, Peer Next Sequence Number: %u\n",
		  apsr_ntohl(ifmp_header->ext.adjacency.peer_ident),
		  apsr_ntohl(ifmp_header->ext.adjacency.peer_next_seq));
          apsr_output("  Reserved(1): %u, Reserved(2): %u\n",
		  apsr_ntohs(ifmp_header->ext.adjacency.res1),
		  ifmp_header->ext.adjacency.res2);
          apsr_output("  Max. Ack Interval: %u\n",
		  ifmp_header->ext.adjacency.max_ack_intvl);
	  break;
	case APSR_IFMP_OPCODE_REDIR:
	case APSR_IFMP_OPCODE_RECLA:
	case APSR_IFMP_OPCODE_RECACK:
	case APSR_IFMP_OPCODE_LABEL:
	case APSR_IFMP_OPCODE_ERROR:
	  apsr_output("  Sender Instance: %u, Peer Instance: %u\n",
	          apsr_ntohl(ifmp_header->ext.redirect.sender_inst),
		  apsr_ntohl(ifmp_header->ext.redirect.peer_inst));
	  apsr_output("  Sequence Number: %u\n",
		  apsr_ntohl(ifmp_header->ext.redirect.seq_num));
	  break;
	default:
	  break; 
   }
	  
  if(statistic) 
          stats.c.ifmp++;
  
  return(RET_SUCCESS);
}
