#include "../include/aprecv-general.h"
#include "../include/aprecv-stream.h"

int print_ipv4_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct ipv4_pkt_header *ipv4_header;
  static struct ipv4_options *ipv4_opt;
  static unsigned short fragment, length, ident, version, header_length, type_service, pkt_checksum, correct_checksum, i;
  char *source_ip, *dest_ip;
  static unsigned char *options=NULL;
  
  ipv4_header = (struct ipv4_pkt_header *)(packet+data_link_size+opt_size);
  
  version 	= APSR_IPV4_VERSION(ipv4_header);
  header_length = APSR_IPV4_HEADERLENGTH(ipv4_header);
  fragment 	= apsr_ntohs(ipv4_header->fragment);
  type_service  = apsr_ntohs(ipv4_header->type_service);
  length	= apsr_ntohs(ipv4_header->packet_len);
  ident		= apsr_ntohs(ipv4_header->ident);
  source_ip 	= strdup(inet_ntoa(ipv4_header->src_ip));
  dest_ip 	= strdup(inet_ntoa(ipv4_header->dst_ip));

  apsr_output("\n- IPv4 Header -\n  Version: %d, Hdr-Len: %d, Length: %d\n",
                                   (int) version,
				   (int) header_length,
				   (int) length);
  apsr_output("  TTL: %d, TOS: 0x%02x", (int) ipv4_header->time_live,
                                   (unsigned int) type_service);
  
  if(verbose) {
	 switch(type_service) {
	 	case APSR_IP_TOS_DEFAULT:
	          apsr_output(" (default");
		  break;
	 	case APSR_IP_TOS_MIN_COS:
		  apsr_output(" (min cost");
	 	  break;
	 	case APSR_IP_TOS_MAX_REL:
	 	  apsr_output(" (max rel.");
	 	  break;
		case APSR_IP_TOS_MAX_THR:
	          apsr_output(" (max thr.");
		  break;
	 	case APSR_IP_TOS_MIN_DEL:
	          apsr_output(" (min del.");
	          break;
	 	case APSR_IP_TOS_MAX_SEC:
	          apsr_output(" (max sec.");
	          break;
	 	default:
		  apsr_output(" (unknown");
	          break;
	}
	if(type_service & 0x03) {
		switch(type_service & 0x03) {
			case 1:
		          apsr_output("|ECT[1]");
			  break;
			case 2:
			  apsr_output("|ECT[0]");
			  break;
			case 3:
			  apsr_output("|CE");
			  break;
		}
	}
	else 
		apsr_output("|Not-ECT");
	apsr_output(")");
  }
		 
  apsr_output(", ID: %u\n  Frag.: 0x%x", ident, fragment);

  if(verbose) {
	  if(fragment & (APSR_IP_NF|APSR_IP_RF|APSR_IP_DF|APSR_IP_MF)) 
		  apsr_output(" (");
	  if(fragment & APSR_IP_NF) 
		  apsr_output("no frag. ");
          if(fragment & APSR_IP_RF)
		  apsr_output("res. frag. ");		  
	  if(fragment & APSR_IP_DF)
		  apsr_output("dont frag. ");
	  if(fragment & APSR_IP_MF) 
		  apsr_output("more frag. ");
	  if(fragment & (APSR_IP_NF|APSR_IP_RF|APSR_IP_DF|APSR_IP_MF))
		  apsr_output(")");
  }
  if((fragment & APSR_IP_OFFMASK) != 0) 
	  apsr_output(" [Offset=%d]", (fragment & APSR_IP_OFFMASK) * 8);
  
/*  if(verbose) {
    	 switch(fragment) {
         	case APSR_IP_NF:
         	  apsr_output(" (no frag.)");
                  break;
          	case APSR_IP_RF:
                  apsr_output(" (res. flag)");
                  break;
         	case APSR_IP_DF:
                  apsr_output(" (dont frag.)");
                  break;
         	case APSR_IP_MF:
                  apsr_output(" (more frag.)");
                  break;
         	case APSR_IP_OFFMASK:
                  apsr_output(" (frag. bit mask)");
                  break;	 
	 	default:
	          break;
	}
  }
*/  
  apsr_output("\n  Src IP: %s", source_ip);
  if(verbose) {
/*	  resolve_ip(source_ip, AF_INET); */
  }
  apsr_output("\n  Dst IP: %s", dest_ip);
  if(verbose) {
/*	  resolve_ip(dest_ip, AF_INET); */
  }
  
  apsr_output("\n  Protocol: %d",(int) ipv4_header->protocol);
 
  /* If verbose than print the name of the protocol */
  
  if(verbose) 
    print_ipv4_proto((unsigned short) ipv4_header->protocol);

  pkt_checksum = apsr_ntohs(ipv4_header->sum);
  apsr_output("\n  Checksum: 0x%04x", pkt_checksum);

  if(calc_checksums) {
    ipv4_header->sum=0x0000;
    correct_checksum=apsr_ntohs(in_cksum((unsigned short *)ipv4_header, 
			    header_length*4, 0));
    if(pkt_checksum == correct_checksum) {
	    apsr_output(" (correct checksum)");
    } 
    else {
	    apsr_output(" (incorrect checksum - should be 0x%04x)", 
			    correct_checksum);

	    if(statistic)
		    stats.cr.ipv4++;
    }

    if(ipv4_header->protocol == APSR_PROTO_UDP) {
      /* Prepare UDP over IPv4 pseudo header */
      memcpy(&v4_ps_hdr.ps_hdr.src_ip, &ipv4_header->src_ip.s_addr,
		    sizeof(unsigned int));
      memcpy(&v4_ps_hdr.ps_hdr.dst_ip, &ipv4_header->dst_ip.s_addr,
		    sizeof(unsigned int));
      v4_ps_hdr.ps_hdr.length = apsr_htons(length-(header_length*4));
    }
    else if(ipv4_header->protocol == APSR_PROTO_TCP) {
      /* Prepare TCP over IPv4 pseudo header */
      memcpy(&v4_ps_hdr.ps_hdr.src_ip, &ipv4_header->src_ip.s_addr,
		    sizeof(unsigned int));
      memcpy(&v4_ps_hdr.ps_hdr.dst_ip, &ipv4_header->dst_ip.s_addr,
		    sizeof(unsigned int));
      v4_ps_hdr.ps_hdr.length = apsr_htons(length-(header_length*4));
    }
  }

  if(header_length > 5) {
	  apsr_output("\n  Options");
	  if(verbose) 
	  	apsr_output("(%u bytes)", (header_length - 5)*4);
	  apsr_output(": [");
	  options=(packet+data_link_size+opt_size+APSR_IPv4_H);
	  for(i=0; i < (header_length-5)*4;) {
		  switch(options[i]) {
			  case APSR_IP_OPT_EOOL:
                            if(verbose)
			           apsr_output(" <End of Option List>");
		 	    else
                                   apsr_output(" <EOOL>");
			    /* To avoid loop */
			    goto unknown;
			    /* Never reached */
			    break;
			  case APSR_IP_OPT_NOP:
			    if(verbose)
			            apsr_output(" <No Operation>");
		            else
			    	    apsr_output(" <NOP>");
			    i++;
			    break;
			  case APSR_IP_OPT_RR:
			    ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
			    apsr_output(" <RR Length=%d>", 
					    (int) ipv4_opt->ext.dummy.length);
			    i=i+ipv4_opt->ext.dummy.length;
			    break;
                          case APSR_IP_OPT_ZSU:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <ZSU Length=%d>", 
					    (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
		          case APSR_IP_OPT_MTUP:
			    ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <MTUP Length=%d>",
                                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_MTUR:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <MTUR Length=%d>",
                                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_ENCODE:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <ENCODE Length=%d>",
                                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
			  case APSR_IP_OPT_TS:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <TS Length=%d>",
                                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_TR:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <TR Length=%d>",
                                           (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_SEC:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <SEC Length=%d>",
                                           (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_LSR:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <LSR Length=%d>",
                                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_ESEC:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <E-SEC Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_CIPSO:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <CIPSO Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_SID:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <SID Length=%d>",
		                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_SSR:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <SSR Length=%d>",
		                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_VISA:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <VISA Length=%d>",
		                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_IMITD:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <IMITD Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_EIP:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <EIP Length=%d>",
		                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_ADDEXT:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <ADDEXT Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_RTRALT:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
			    if(verbose)
			            apsr_output(" <Router Alert Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
			    else
			            apsr_output(" <RA Length=%d>",
				              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_SDB:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <SDB Length=%d>",
		                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_NSAPA:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <NSAPA Length=%d>",
		                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_DPS:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <DPS Length=%d>",
		                              (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_UMP:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <UMP Length=%d>",
		                            (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
                          case APSR_IP_OPT_FINN:
                            ipv4_opt=(struct ipv4_options *)(packet+data_link_size+opt_size+APSR_IPv4_H+i);
                            apsr_output(" <FINN Length=%d>",
		                             (int) ipv4_opt->ext.dummy.length);
                            i=i+ipv4_opt->ext.dummy.length;
                            break;
			  default:
			    apsr_output(" <Unknown>");
			    goto unknown;
			    /* Never reached */
			    break;
		  }
          }
unknown:
          apsr_output(" ]");
  }
  else if (header_length < 5) {
	  apsr_output("\n  Error: bad header length");
	  
	  if(statistic)
		  stats.err.ipv4++;
	  
	  goto end_ipv4;
  }

  if(ipv4_header->protocol != APSR_PROTO_IPV6)
	  apsr_output("\n");
	 
  /* Switch to next higher protocol */
 
  if((fragment & APSR_IP_OFFMASK) == 0) {
        if(ipv4_header->protocol == APSR_PROTO_TCP && print_tcp_stream) {
		temp_ipv4_stream_header = ipv4_header;
	}
	switchto_ipv4_proto(header, packet, opt_size+header_length*4, ipv4_header->protocol);
  }
       	
  if(print_payload_ip_hex && !print_payload_tcp_text && !print_payload_tcp_hex && !print_payload_udp_text && !print_payload_udp_hex) {
	  apsr_output("\n");
	  print_raw_in_hex(packet+data_link_size+sizeof(struct ipv4_pkt_header)+opt_size, header->len-sizeof(struct ipv4_pkt_header)-data_link_size-opt_size-1); 
  }

end_ipv4:
  
  if(statistic) 
          stats.c.ipv4++;

  free(source_ip);
  free(dest_ip);

  return(RET_SUCCESS);
}

void print_ipv4_proto(unsigned short protocol) {
  switch(protocol) {
        case APSR_PROTO_IP:
          apsr_output(" (IPv4)");
          break;
	case APSR_PROTO_IP_IN_IP:
	  apsr_output(" (IPv4 in IPv4 Encapsulation)");
	  break;
	case APSR_PROTO_IPV6:
	  apsr_output(" (IPv6 over IPv4)");
	  break;
        case APSR_PROTO_ICMP:
          apsr_output(" (ICMP)");
          break;
        case APSR_PROTO_IGMP:
          apsr_output(" (IGMP)");
          break;
        case APSR_PROTO_TCP:
          apsr_output(" (TCP)");
          break;
        case APSR_PROTO_UDP:
          apsr_output(" (UDP)");
          break;
        case APSR_PROTO_RDP:
          apsr_output(" (RDP)");
          break;
        case APSR_PROTO_IPV6_AUTH:
          apsr_output(" (Authentication header)");
          break;
        case APSR_PROTO_OSPF:
          apsr_output(" (OSPF)");
          break;
        case APSR_PROTO_EGP:
          apsr_output(" (EGP)");
          break;
	case APSR_PROTO_GGP:
          apsr_output(" (GGP)");
          break;
        case APSR_PROTO_IPCOMP:
          apsr_output(" (IPComp)");
          break;
	case APSR_PROTO_IPV6_CRYPT:
	  apsr_output(" (ESP)");
	  break;
        case APSR_PROTO_NARP:
          apsr_output(" (NARP)");
          break;
        case APSR_PROTO_IGRP:
          apsr_output(" (IGRP)");
          break;
        case APSR_PROTO_EIGRP:
          apsr_output(" (EIGRP)");
          break;
	case APSR_PROTO_SCTP:
	  apsr_output(" (SCTP)");
	  break;
	case APSR_PROTO_PIM:
	  apsr_output(" (PIM)");
	  break;
	case APSR_PROTO_GRE:
	  apsr_output(" (GRE)");
	  break;
	case APSR_PROTO_IFMP:
          apsr_output(" (IFMP)");
          break;
	case APSR_PROTO_VRRP:
	  apsr_output(" (VRRP)");
	  break;
	case APSR_PROTO_IRTP:
	  apsr_output(" (IRTP)");
	  break;
	case APSR_PROTO_CBT:
	  apsr_output(" (CBT)");
	  break;
	case APSR_PROTO_RSVP:
	  apsr_output(" (RSVP)");
	  break;
        case APSR_PROTO_ETHERIP:
	  apsr_output(" (EtherIP)");
	  break;
	case APSR_PROTO_MTP:
	  apsr_output(" (MTP)");
	  break;
	case APSR_PROTO_PGM:
	  apsr_output(" (PGM)");
	  break;
	case APSR_PROTO_NETBLK:
          apsr_output(" (NETBLK)");
          break;	  
        case APSR_PROTO_TMUX:
          apsr_output(" (TMux)");
          break;	  
	case APSR_PROTO_SDRP:
	  apsr_output(" (SDRP)");
	  break;
	default:
          apsr_output(" (unknown)");
          break;
  }
}

void switchto_ipv4_proto(struct aprecv_frame_header *header, unsigned char *packet, int opt_size, unsigned short protocol) {
  switch(protocol) {
	    case APSR_PROTO_IP:
	    case APSR_PROTO_IP_IN_IP:
              print_ipv4_packet(header, packet, opt_size);
              break;
	    case APSR_PROTO_IPV6:
	      print_ipv6_packet(header, packet, opt_size);
	      break;
            case APSR_PROTO_UDP:
              print_udp_packet(header, packet, opt_size, 4);
              break;
            case APSR_PROTO_TCP:
              print_tcp_packet(header, packet, opt_size, 4);
              break;
            case APSR_PROTO_ICMP:
              print_icmpv4_packet(header, packet, opt_size);
              break;
            case APSR_PROTO_IGMP:
              print_igmp_packet(header, packet, opt_size);
              break;
            case APSR_PROTO_IPV6_AUTH:
              print_ipah_packet(header, packet, opt_size);
              break;
            case APSR_PROTO_IPV6_CRYPT:
              print_ipesp_packet(packet, opt_size);
              break;
	    case APSR_PROTO_OSPF:
              print_ospf_packet(packet, opt_size);
              break;
            case APSR_PROTO_EGP:
              print_egp_packet(header, packet, opt_size);
              break;
	    case APSR_PROTO_GGP:
              print_ggp_packet(packet, opt_size);
              break;
            case APSR_PROTO_RDP:
              print_rdp_packet(packet, opt_size);
              break;
            case APSR_PROTO_IPCOMP:
              print_ipcomp_packet(packet, opt_size);
              break;
            case APSR_PROTO_NARP:
              print_narp_packet(packet, opt_size);
              break;
            case APSR_PROTO_IGRP:
              print_igrp_packet(header, packet, opt_size);
              break;
            case APSR_PROTO_EIGRP:
              print_eigrp_packet(header, packet, opt_size);
              break;
	    case APSR_PROTO_SCTP:
	      print_sctp_packet(packet, opt_size);
	      break;
	    case APSR_PROTO_PIM:
	      print_pim_packet(packet, opt_size);
	      break;
	    case APSR_PROTO_GRE:
	      print_gre_packet(header, packet, opt_size);
	      break;
	    case APSR_PROTO_IFMP:
              print_ifmp_packet(packet, opt_size);
              break;
	    case APSR_PROTO_VRRP:
	      print_vrrp_packet(packet, opt_size);
	      break;
	    case APSR_PROTO_IRTP:
	      print_irtp_packet(header, packet, opt_size);
	      break;
	    case APSR_PROTO_CBT:
	      print_cbt_packet(header, packet, opt_size);
	      break;
	    case APSR_PROTO_RSVP:
	      print_rsvp_packet(header, packet, opt_size);
	      break;
	    case APSR_PROTO_ETHERIP:
	      print_etherip_packet(header, packet, opt_size);
	      break;
	    case APSR_PROTO_MTP:
	      print_mtp_packet(packet, opt_size);
	      break;
	    case APSR_PROTO_PGM:
	      print_pgm_packet(packet, opt_size);
	      break;
            case APSR_PROTO_NETBLK:
              print_netblk_packet(packet, opt_size);
              break;	      
            case APSR_PROTO_TMUX:
              print_tmux_mini_packet(packet, opt_size);
              break;	      
	    default:
              break;
  }
}
