#include "../include/aprecv-general.h"

static const char *mopv3_prgtype[]= {
	" (Secondary Loader)",
	" (Tertiary Loader)",
	" (Operation System)",
	" (Management Image)",
	" (CMIP Script File)",
};

int print_mopv3_dumpload_packet (unsigned char *packet, int opt_size) {
  static struct mopv3_dumpload_rqprg_pkt_header *mopv3_dumpload_rqprg_header;
  static struct mopv3_dumpload_rqmemld_pkt_header *mopv3_dumpload_rqmemld_header;
  static struct mopv3_dumpload_memld_pkt_header *mopv3_dumpload_memld_header;
  static struct mopv3_dumpload_parldta_pkt_header *mopv3_dumpload_parldta_header;

  mopv3_dumpload_rqprg_header = (struct mopv3_dumpload_rqprg_pkt_header *)(packet+data_link_size+opt_size);

  apsr_output("\n- MOPv3 Dump/Load Protocol header -\n");
  apsr_output("  Code: %u", mopv3_dumpload_rqprg_header->code);
  if(verbose) {
	  switch(mopv3_dumpload_rqprg_header->code) {
		  case APSR_MOPV3_CODE_MEMLDTA:
		    apsr_output(" (Memory Load with Transfer Address)");
		    break;
                  case APSR_MOPV3_CODE_DMPCMPL:
                    apsr_output(" (Dump Complete)");
                    break;
                  case APSR_MOPV3_CODE_MEMLOAD:
                    apsr_output(" (Memory Load)");
                    break;
                  case APSR_MOPV3_CODE_ASSIVOL:
                    apsr_output(" (Assistance volunteer)");
                    break;
                  case APSR_MOPV3_CODE_RQMEMDP:
                    apsr_output(" (Request Memory Dump)");
                    break;
		  case APSR_MOPV3_CODE_REQSTID:
		    apsr_output(" (Request ID)");
		    break;
		  case APSR_MOPV3_CODE_BOOT:
		    apsr_output(" (Boot)");
		    break;
		  case APSR_MOPV3_CODE_SYSID:
		    apsr_output(" (System ID)");
		    break;
                  case APSR_MOPV3_CODE_RQPROG:
                    apsr_output(" (Request Program)");
                    break;
		  case APSR_MOPV3_CODE_RQCOUNT:
		    apsr_output(" (Request Counters)");
		    break;
                  case APSR_MOPV3_CODE_RQMEMLD:
                    apsr_output(" (Request Memory Load)");
                    break;
		  case APSR_MOPV3_CODE_COUNTER:
		    apsr_output(" (Counters)");
		    break;
                  case APSR_MOPV3_CODE_RQDMPSER:
                    apsr_output(" (Request Dump Service)");
                    break;
                  case APSR_MOPV3_CODE_MEMDMPDT:
                    apsr_output(" (Memory Dump Data)");
                    break;
                  case APSR_MOPV3_CODE_PARLDATA:
                    apsr_output(" (Parameter Load with Transfer Address)");
                    break;
		  case APSR_MOPV3_CODE_RESCONS:
		    apsr_output(" (Reserve Console)");
		    break;
		  case APSR_MOPV3_CODE_REALCON:
		    apsr_output(" (Release Console)");
		    break;
		  case APSR_MOPV3_CODE_RESERVED1:
		  case APSR_MOPV3_CODE_RESERVED2:
		    apsr_output(" (Reserved)");
		    break;
		  case APSR_MOPV3_CODE_CCP:
		    apsr_output(" (Console Command and Poll)");
		    break;
		  case APSR_MOPV3_CODE_CRA:
		    apsr_output(" (Console Response and Ack)");
		    break;
		  case APSR_MOPV3_CODE_COUNTERS:
		    apsr_output(" (Counters)");
		    break;
		  case APSR_MOPV3_CODE_ALD:
		    apsr_output(" (Active Loop Data)");
		    break;
		  case APSR_MOPV3_CODE_PLD:
		    apsr_output(" (Passive Loop Data)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	  }
  }		    
  apsr_output("\n");

  switch(mopv3_dumpload_rqprg_header->code) {
	  case APSR_MOPV3_CODE_RQPROG:
	    apsr_output("  Device Type: %u, Fmt. Version: %u\n",
			    mopv3_dumpload_rqprg_header->dev_type,
			    mopv3_dumpload_rqprg_header->format_version);
	    apsr_output("  Program Type: %u",
			    mopv3_dumpload_rqprg_header->prog_type);
            if(verbose) 
		    apsr_output("%s", mopv3_prgtype[mopv3_dumpload_rqprg_header->prog_type]);
	    apsr_output("\n");
	    break;
	  case APSR_MOPV3_CODE_RQMEMLD:
	    mopv3_dumpload_rqmemld_header = 
		    (struct mopv3_dumpload_rqmemld_pkt_header *)(packet+data_link_size+opt_size);
	    apsr_output("  Load Number: %u, Error: %u\n",
			    mopv3_dumpload_rqmemld_header->load_num,
			    mopv3_dumpload_rqmemld_header->error);
	    break;
	  case APSR_MOPV3_CODE_MEMLOAD:
            mopv3_dumpload_memld_header =
	            (struct mopv3_dumpload_memld_pkt_header *)(packet+data_link_size+opt_size);
	    apsr_output("  Load Number: %u, Load Address: 0x%02x 0x%02x 0x%02x 0x%02x\n",
			    mopv3_dumpload_memld_header->load_num,
			    mopv3_dumpload_memld_header->load_addr[0],
			    mopv3_dumpload_memld_header->load_addr[1],
			    mopv3_dumpload_memld_header->load_addr[2],
			    mopv3_dumpload_memld_header->load_addr[3]);
	    break;
	  case APSR_MOPV3_CODE_PARLDATA:
	    mopv3_dumpload_parldta_header =
		    (struct mopv3_dumpload_parldta_pkt_header *)(packet+data_link_size+opt_size);
	    apsr_output("  Load Number: %u, Parameter: %u\n",
			    mopv3_dumpload_parldta_header->load_num,
			    mopv3_dumpload_parldta_header->para);
	    apsr_output("  Transfer Address: 0x%02x 0x%02x 0x%02x 0x%02x\n",
			    mopv3_dumpload_parldta_header->trans_addr[0],
			    mopv3_dumpload_parldta_header->trans_addr[1],
			    mopv3_dumpload_parldta_header->trans_addr[2],
			    mopv3_dumpload_parldta_header->trans_addr[3]);
	    break;
	  default:
	    break;
  }
			    
  if(statistic) 
        stats.c.mopv3_dumpload++;
 
  return(RET_SUCCESS);
}
