#include "../include/aprecv-general.h"

int print_sll_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct sll_pkt_header *sll_header;
  #ifdef APRECV_DEBUG
  static unsigned short pkt_type, ll_type, ll_length;
  #endif
	  
  static unsigned short protocol;
  
  sll_header = (struct sll_pkt_header *)(packet+data_link_size+opt_size);
  protocol = apsr_ntohs(sll_header->sll_protocol);
  
  #ifdef APRECV_DEBUG
  pkt_type = apsr_ntohs(sll_header->sll_pkt_type);
  
  apsr_output("\n- SLL Header -\n  Packet type: %d", pkt_type);
  if(verbose) {
	  switch(pkt_type) {
		  case APSR_SLL_PKTTYPE_HOST:
		    apsr_output(" (Host)");
		    break;
		  case APSR_SLL_PKTTYPE_BROAD:
		    apsr_output(" (Broadcast)");
		    break;
		  case APSR_SLL_PKTTYPE_MULTI:
		    apsr_output(" (Multicast)");
		    break;
		  case APSR_SLL_PKTTYPE_OTHER:
		    apsr_output(" (Otherhost)");
		    break;
		  case APSR_SLL_PKTTYPE_OUTGO:
		    apsr_output(" (Outgoing)");
		    break;
		  default:
		    apsr_output(" (Unknown)");
		    break;
 	  }		  
  }			  

  ll_length = apsr_ntohs(sll_header->sll_lla_len);
  ll_type   = apsr_ntohs(sll_header->sll_lla_type);
  apsr_output("\n  Link-Layer type: 0x%04x",
		  ll_type);
  if(verbose) {
	  if(ll_type == APSR_SLL_LLTYPE_ETHER) 
		  apsr_output(" (Ethernet)");
	  else if(ll_type == APSR_SLL_LLTYPE_LOOP)
		  apsr_output(" (Loopback)");
  }
  
  apsr_output(", Link-Layer length: %d\n",
		  ll_length);
  
  if(ll_length == 6) { /* We assume that 6 is ethernet */
	  apsr_output("  Link-Layer Address: %s", 
			  ether2str(sll_header->sll_addr));
   	  if(verbose) 
		  resolve_mac(sll_header->sll_addr);
  }
  else    
	  apsr_output("  Link-Layer Address: %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
		  sll_header->sll_addr[0], sll_header->sll_addr[1],
		  sll_header->sll_addr[2], sll_header->sll_addr[3],
		  sll_header->sll_addr[4], sll_header->sll_addr[5],
		  sll_header->sll_addr[6], sll_header->sll_addr[7]);
  apsr_output("\n  Protocol: 0x%04x", protocol);
  if(verbose)
	  print_ethernet_proto(protocol);
  #endif
  
  /* No statistic count for now. */

  opt_size += 16; /* Entire SLL header */
  
  if(protocol <= 1536) {
	  switch(protocol) {
		  case APSR_LINUX_SLL_802_2: /* LLC */
		    print_llc_packet(header, packet, opt_size, protocol);
                    break;	
                  case APSR_LINUX_SLL_802_3:
		    print_ipx_packet(packet, opt_size);
		    break;
		  default:
		    break;
	  }
  }
  else 
	  switchto_ethernet_proto(header, packet, opt_size, protocol);
  
  return(RET_SUCCESS);
}
