#include "../include/aprecv-general.h"

#ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
int start_sniffing_bluetooth(char *arg, int device) {
  /* Check user ID */
  apsr_check_uid();

  /* Open a link layer interface. */
  apsrlib_bluetooth_fd = apsr_open_bluetooth_socket(device);
  if(apsrlib_bluetooth_fd == -1) {
	  exit(EXIT_FAILURE);
  }

  apsr_output("Receive packet(s) on device: hci%d\n", device);
  
  apsr_read_bluetooth_socket(apsrlib_bluetooth_fd, 1024, 0, (apsr_read_bluetooth_callback)recv_packet_bluetooth);

  return(RET_SUCCESS);
}

int recv_packet_bluetooth(unsigned char *packet, unsigned int len) {
  struct timeval ts;
  static unsigned int sec;
  static unsigned long diff_receive_time;
  static float utilization;
  time_t now_receive_time;
  struct aprecv_frame_header aprecv_header;
   
  stats.c.all++;
  
  if(beep_on)
	  apsr_output("\a");

  all_counted_bytes += len;

  /* print out the time when the packet comes in & utilization */
  /* FIXME: captured length/len */
  if(gettimeofday(&(ts), NULL) < 0) {
	  apsr_stream_output((FILE *) stderr, "Error: Can't get time.\n");
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "Function gettimeofday failed while receive packet, error code: %s.\n", strerror(errno));
	  #endif
  }
  sec = ts.tv_sec % 86400;
  apsr_output("\n%02d:%02d:%02d.%06u - Received frame %lu(%u bytes length)...", sec / 3600, (sec % 3600) / 60, sec % 60, ts.tv_usec, stats.c.all, len);

  if(statistic) {
	  now_receive_time = time(NULL);
	  diff_receive_time = difftime(now_receive_time, start_receive_time);
	  if(diff_receive_time > 0) {	/* Needed, overwhise core dump */
		  utilization = all_counted_bytes / diff_receive_time;
                  if(utilization > 1024000) { /* MB */
                          utilization = utilization / 1024000;
                          apsr_output(" Utilization: %.0f MB/s", utilization);
                  }
                  if(utilization > 1024) { /* KB */
                          utilization = utilization / 1024;
                          apsr_output(" Utilization: %.0f KB/s", utilization);
                  }
                  else
                          apsr_output(" Utilization: %.0f B/s", utilization);
          }
	  else {
		  apsr_output(" Utilization: n.a.");
          }
  }
  apsr_output("\n");

  /* Map to aprecv_frame_header */
  aprecv_header.caplen = len; /* FIXME see above */
  aprecv_header.len    = len;
  
  if(!no_dissection)
	  print_hci_packet(&aprecv_header, packet, 0);
  
  if(server_addr)
	  send_server_got_packet(packet, len);
	 
  if(fflush(stdout) == EOF) {
 	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "Error: fflush(stdout) %s.\n", strerror(errno));
	  #endif
  }
 
  return(RET_SUCCESS);  
}
#endif
