#include "../include/apsrstack-general.h"

void process_tcp_packet (struct apsrstack_frame_header *header, void *packet, int opt_size) {
  struct ipv4_pkt_header *ipv4_header;
  struct tcp_pkt_header *tcp_header;
  struct tcp_option_mss tcp_mss;
  unsigned short c = 0, x = 0, calc_sum = 0, payload_length = 0;
  unsigned short tcp_payload_length = 0;
  unsigned int seq_num = 0, ack_num = 0, seq_ack_difference = 0;
  unsigned char flags = 0;
  unsigned char *tcp_payload;
  struct timeval tv;
  
  ipv4_header = (struct ipv4_pkt_header *)(packet+data_link_size);
  tcp_header = (struct tcp_pkt_header *)(packet+data_link_size+opt_size);

  payload_length = apsr_ntohs(ipv4_header->packet_len) - (APSR_IPV4_HEADERLENGTH(ipv4_header)*4);    
  seq_num = apsr_ntohl(tcp_header->seq_num);
  ack_num = apsr_ntohl(tcp_header->ack_num);

  /*
   * Check for wrong flags.
   */
  if(tcp_header->flags & APSR_TCP_SYN && tcp_header->flags & APSR_TCP_RST &&
     tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_FIN &&
     tcp_header->flags & APSR_TCP_ACK && tcp_header->flags & APSR_TCP_URG) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Drop TCP packet, because ALL flags set.\n");
    #endif
    stats.tcp.drops++; /* Drops++ */
    return;
  }
  else if(tcp_header->flags & APSR_TCP_SYN && 
          tcp_header->flags & APSR_TCP_RST) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Drop TCP packet, because SYN+RST set.\n");
    #endif
    stats.tcp.drops++; /* Drops++ */	  
    return;
  }
  else if(tcp_header->flags == 0) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Drop TCP packet, because no flags set.\n");
    #endif
    stats.tcp.drops++; /* Drops++ */
    return;
  }   
  
  /*
   * Checksum verification.
   */
  memcpy(&v4_ps_hdr.ps_hdr.src_ip, &ipv4_header->src_ip.s_addr,
		  sizeof(unsigned int));
  memcpy(&v4_ps_hdr.ps_hdr.dst_ip, &ipv4_header->dst_ip.s_addr,
                  sizeof(unsigned int));
  v4_ps_hdr.ps_hdr.length = apsr_htons(apsr_ntohs(ipv4_header->packet_len)-(APSR_IPV4_HEADERLENGTH(ipv4_header)*4));
  calc_sum = apsr_tcpv4_checksum(tcp_header);
  if(calc_sum != 0) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Got TCP packet with wrong checksum.\n");
    #endif
    stats.tcp.checksum_errors++; /* Checksum Errors++*/
    return;
  }

  /* 
   * Check if a apsrstack_listen is on this port 
   */
  for(c = 0; c < APSRSTACK_TCP_MAX_PORTS; c++) {
    if(tcp_listen_ports[c] == tcp_header->dst_port) {
      goto check_connection_state;
      break; /* Never reached */      
    }      
  }  
  
  /* 
   * There is no listen on this port, so we send a RST/ACK packet 
   */
  flags |= APSR_TCP_RST;
  flags |= APSR_TCP_ACK;
  send_tcp(header,
 	   5,
           APSR_TCP,
	   apsr_ntohs(ipv4_header->ident),
	   ipv4_header->dst_ip, /* Our IP Address */
	   ipv4_header->src_ip,
	   APSRSTACK_IPV4_DEFAULT_TTL,
	   apsr_ntohs(tcp_header->dst_port),
	   apsr_ntohs(tcp_header->src_port),
	   0,
	   seq_num+1, 
	   flags,
	   5,
	   0,
	   0,
	   0,
	   0,
	   NULL,
	   0);
  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: SYN try to closed port, RST/ACK send.\n");
  #endif
  stats.tcp.processed++; /* Processed++ */
  return;

check_connection_state:
  for(x = 0; x < APSRSTACK_TCP_MAX_CONNECTIONS; x++) {
    if(tcp_queue[x].state != APSRSTACK_TCP_STATE_LISTEN &&
      tcp_queue[x].peer_ip.s_addr == ipv4_header->src_ip.s_addr &&
      apsr_htons(tcp_queue[x].local_port) == tcp_listen_ports[c] &&
      apsr_htons(tcp_queue[x].peer_port) == tcp_header->src_port) {
      goto connection_known;
      break; /* Never reached */		  
    }
  }
  
  /*
   * Connection is not open. Check flags.
   */

  if(tcp_header->flags & APSR_TCP_SYN &&
     tcp_header->flags & APSR_TCP_ECE) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Got packet with SYN/ECE set, ECN is currently not implemented.\n");
    #endif 
  }
  
  if(tcp_header->flags == APSR_TCP_SYN) 
	  goto open_connection;
 
  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Got packet with no SYN set, drop it.\n");
  #endif
  stats.tcp.drops++; /* Drops++ */
  return;
  
open_connection:
  /*
   * Search for free place in queue, and fill it.
   */
  for(x = 0; x < APSRSTACK_TCP_MAX_CONNECTIONS; x++) {
    if(tcp_queue[x].state == APSRSTACK_TCP_STATE_LISTEN) {
      tcp_queue[x].state = APSRSTACK_TCP_STATE_SYN_RECEIVED;
      tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
      tcp_queue[x].peer_ip.s_addr = ipv4_header->src_ip.s_addr;
      tcp_queue[x].local_port = apsr_ntohs(tcp_header->dst_port);
      tcp_queue[x].peer_port = apsr_ntohs(tcp_header->src_port);
      tcp_queue[x].seq_next = 0;
      tcp_queue[x].ack_next = seq_num + 1;
      goto send_syn_ack;
      break; /* Never reached */		  
    }
  }

  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Queue is full!\n");
  #endif
  stats.tcp.drops++; /* Drops */
  return;

send_syn_ack:
  tcp_payload = xmalloc(4);
  
  tcp_mss.option = 2;
  tcp_mss.length = 4;
  tcp_mss.mss    = apsr_htons(APSRSTACK_TCP_DEFAULT_MSS);

  tcp_payload = (unsigned char *)&tcp_mss;
 
  tcp_payload_length = sizeof(tcp_payload);

  flags |= APSR_TCP_SYN;
  flags |= APSR_TCP_ACK;
  send_tcp(header,
           5,
           APSR_TCP+4,
           apsr_ntohs(ipv4_header->ident),
           ipv4_header->dst_ip,	/* Our IP Address */
           tcp_queue[x].peer_ip,
           APSRSTACK_IPV4_DEFAULT_TTL,
           tcp_queue[x].local_port,
           tcp_queue[x].peer_port,
           tcp_queue[x].seq_next,
           tcp_queue[x].ack_next,
           flags,
           6,
           0,
           0,
           APSRSTACK_TCP_DEFAULT_WINDOW_SIZE,
           0,
           tcp_payload,
           tcp_payload_length);
  
  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Connection state from LISTEN -> SYN-RECEIVED, SYN/ACK send.\n");
  #endif
  stats.tcp.processed++; /* Processed++ */  
  return;
    
connection_known:
  /*
   * Check if connection is in state SYN-RECEIVED or ESTABLISHED.
   */
  if(tcp_queue[x].state == APSRSTACK_TCP_STATE_SYN_RECEIVED) {
    goto check_syn_received;
  }
  else if(tcp_queue[x].state == APSRSTACK_TCP_STATE_ESTABLISHED) {
    goto check_established;
  }
  else if(tcp_queue[x].state == APSRSTACK_TCP_STATE_LAST_ACK) {
    goto check_last_ack;
  }

  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Connection state not implemented.\n");
  #endif
  stats.tcp.drops++; /* Drops++ */      
  return;

check_syn_received:
  /*
   * Check if the next packet after SYN-RECEIVED is an ACK.
   */
  if(tcp_header->flags == APSR_TCP_ACK &&
     ack_num == tcp_queue[x].seq_next + 1) {
    tcp_queue[x].seq_next = tcp_queue[x].seq_next + 1;
    tcp_queue[x].state = APSRSTACK_TCP_STATE_ESTABLISHED;
    tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Connection state from SYN-RECEIVED -> ESTABLISHED, ACK received.\n");
    #endif
    stats.tcp.processed++; /* Processed++ */
    return;
  }
  /*
   * Check if the next packet after SYN-RECEIVED is a RST.
   */
  else if(tcp_header->flags == APSR_TCP_RST &&
	  ack_num == tcp_queue[x].seq_next + 1) {
    tcp_queue[x].state = APSRSTACK_TCP_STATE_LISTEN;
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Connection state from SYN-RECEIVED -> LISTEN, RST received.\n");
    #endif 
    stats.tcp.processed++; /* Processed++ */
    return;
  }	  
  else {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Got unknown packet in connection state SYN-RECEIVED.\n");
    #endif
    stats.tcp.drops++; /* Drops++ */    
    return;
  }    

check_established:
  if(tcp_header->flags & APSR_TCP_ACK) {
    seq_ack_difference = ack_num - tcp_queue[x].seq_next;
    if(seq_ack_difference > 0) {
       tcp_queue[x].seq_next = tcp_queue[x].seq_next + seq_ack_difference;
    }
    tcp_queue[x].state = APSRSTACK_TCP_STATE_ESTABLISHED;
    tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Connection state is ESTABLISHED, process data.\n");
    #endif
    goto process_data;
    return;
  }
  else {
    stats.tcp.drops++; /* Drops++ */
    return; 
  }

check_last_ack:
  if(tcp_header->flags & APSR_TCP_ACK) {
    if(ack_num == (tcp_queue[x].seq_next + 1)) {
      tcp_queue[x].state = APSRSTACK_TCP_STATE_CLOSED;
      tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
      #ifdef APSRSTACK_DEBUG
      apsr_output("DEBUG: Connection state from LAST-ACK to CLOSED.\n");
      #endif

      tcp_queue[x].state = APSRSTACK_TCP_STATE_LISTEN;
      tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
      #ifdef APSRSTACK_DEBUG
      apsr_output("DEBUG: Connection state from CLOSED to LISTEN, make it free.\n");
      #endif
      stats.tcp.processed++; /* Processed++ */
      return;
    }
    return;
  }
  else {
    stats.tcp.drops++; /* Drops++ */
    return;
  }

process_data:
  seq_ack_difference = tcp_queue[x].ack_next - seq_num;

  if (seq_ack_difference >= 0) {
    tcp_queue[x].ack_next = tcp_queue[x].ack_next + payload_length;
    tcp_queue[x].ack_next = tcp_queue[x].ack_next - tcp_header->data_offset * 4;
    
    if(tcp_header->flags & APSR_TCP_FIN) {

      /* First go to CLOSE-WAIT */

      tcp_queue[x].state = APSRSTACK_TCP_STATE_CLOSE_WAIT;
      tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;
      tcp_queue[x].ack_next = tcp_queue[x].ack_next + 1;
                                                                                
      flags |= APSR_TCP_ACK;
      send_tcp(header,
               5,
               APSR_TCP,
               apsr_ntohs(ipv4_header->ident),
               ipv4_header->dst_ip, /* Our IP Address */
               tcp_queue[x].peer_ip,
               APSRSTACK_IPV4_DEFAULT_TTL,
               tcp_queue[x].local_port,
               tcp_queue[x].peer_port,
               tcp_queue[x].seq_next,
               tcp_queue[x].ack_next,
               flags,
               5,
               0,
               0,
               0,
               0,
               NULL,
               0);
                                                                                
      #ifdef APSRSTACK_DEBUG
      apsr_output("DEBUG: Connection state from ESTABLISHED to CLOSE-WAIT, send ACK.\n");
      #endif
                                                                                
      /* And now go to LAST-ACK */

      tcp_queue[x].state = APSRSTACK_TCP_STATE_LAST_ACK;
      tcp_queue[x].timeout = APSRSTACK_TCP_STATES_TIMEOUT;

      flags |= APSR_TCP_ACK;
      flags |= APSR_TCP_FIN;
      send_tcp(header,
               5,
               APSR_TCP,
               apsr_ntohs(ipv4_header->ident),
               ipv4_header->dst_ip, /* Our IP Address */
               tcp_queue[x].peer_ip,
               APSRSTACK_IPV4_DEFAULT_TTL,
               tcp_queue[x].local_port,
               tcp_queue[x].peer_port,
               tcp_queue[x].seq_next,
               tcp_queue[x].ack_next,
               flags,
               5,
               0,
               0,
               0,
               0,
               NULL,
               0);
                                                                                
      #ifdef APSRSTACK_DEBUG
      apsr_output("DEBUG: Connection state from CLOSE-WAIT to LAST_ACK, send FIN.\n");
      #endif
                                                                                
      stats.tcp.processed++; /* Processed++ */
      return;
    }
    return;
  }
  /* TODO */
  else {
    stats.tcp.drops++; /* Drops++ */
    return;
  }
}

/* Send raw tcp packets, without payload */
void send_tcp(struct apsrstack_frame_header *header, 
	      unsigned char ip_hdr_length,
	      unsigned char ip_length, unsigned short ip_ident,
	      struct in_addr ip_src, struct in_addr ip_dst,
	      unsigned char ip_ttl, unsigned short tcp_sport,
	      unsigned short tcp_dport, unsigned int tcp_seqnum,
	      unsigned int tcp_acknum, unsigned char tcp_flags, 
	      unsigned char tcp_offset, unsigned char tcp_reserved1, 
	      unsigned char tcp_reserved2, unsigned short tcp_window, 
	      unsigned short tcp_urg, unsigned char *tcp_payload, 
	      unsigned short tcp_payload_length) {	      

	struct apsr_link_layer *apsr_ll;
        unsigned char *buf;
  
        apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

        apsr_init_psize(APSR_ETH + (ip_hdr_length * 4) + APSR_TCP + tcp_payload_length, &buf);

	apsr_ethernet(header->dst_mac, header->src_mac, header->type, NULL, 0, buf);

        apsr_ipv4(4, ip_hdr_length, ip_length, 0x0000, ip_ident, 0x0000, ip_ttl, APSR_PROTO_TCP, ip_src.s_addr, ip_dst.s_addr, 0, NULL, 0, NULL, 0, buf + APSR_ETH);
	apsr_tcp(tcp_sport, tcp_dport, tcp_seqnum, tcp_acknum, tcp_flags, tcp_offset, tcp_reserved1, tcp_reserved2, tcp_window, tcp_urg, 0, tcp_payload, tcp_payload_length, buf + APSR_ETH + APSR_IPv4);  
	
        apsr_checksum(buf + APSR_ETH, APSR_PROTO_TCP, APSR_TCP + tcp_payload_length);
 
 	apsr_write_link_layer(apsr_ll, device, buf, APSR_ETH + (ip_hdr_length * 4) + APSR_TCP + tcp_payload_length, 1);

	apsr_close_link_layer(apsr_ll);
	free(buf);
}

	
