#ifndef APSR_ETHER_ADDR_LEN
  #define APSR_ETHER_ADDR_LEN 6
#endif

#ifndef APSR_TOKEN_ADDR_LEN
  #define APSR_TOKEN_ADDR_LEN	6
#endif

#ifndef APSR_FDDI_ADDR_LEN
  #define APSR_FDDI_ADDR_LEN 	6
#endif

/* Link layer interface struct. */
struct apsr_link_layer {
	int fd;
	int type;
	int offset;
	unsigned char *device;
};

/* APSRSERVER<->APRECV protocol */
struct apsrserver_protocol_header {
	unsigned int version:8;		/* Version	*/
	unsigned short id;		/* ID		*/
	unsigned int command:8;		/* Command	*/
	unsigned int code:8;		/* Code		*/
	unsigned short length;		/* Length	*/
};

/* Mac adr */
struct mac_addr {
  unsigned char ether_addr_octet[APSR_ETHER_ADDR_LEN];
};

/* Ethernet header */
/* Checksum is filled up by the kernel. */
struct ethernet_pkt_header {
  unsigned char destination[APSR_ETHER_ADDR_LEN];
  unsigned char source[APSR_ETHER_ADDR_LEN];
  unsigned short type;
  union {
	  struct {
		  unsigned char dsap;          	/* DSAP (LLC)		     */
		  unsigned char ssap;          	/* SSAP (LLC)		     */
		  unsigned char control;       	/* Control (LLC)	     */
		  unsigned char org_id[2];     	/* Org ID/OUI(3 bytes)	     */
		  unsigned short type;         	/* Type, same as Ethernet II */
	  } snap;				/* 802.3 LLC + SNAP */
  } ext;
};

/* LLC header */
struct llc_pkt_header {
  unsigned char dsap;          /* DSAP (LLC)			*/
  unsigned char ssap;          /* SSAP (LLC)			*/
  unsigned char control;       /* Control (LLC)			*/
  unsigned char org_id[2];     /* Org ID/OUI(3 bytes)		*/
  unsigned short type;         /* Type, same as Ethernet II	*/
};

/* VLAN 802.1q header */
struct vlan_pkt_header {
  unsigned short vlan;		/* Priority/CFI/VLAN-ID */
  unsigned short type;
};

/* IEEE 802.11 header */
struct wvlan_pkt_header {
  unsigned short fc;				/* Frame Control	*/
  union {
  	struct {
		unsigned short duration;        /* Duration / ID	*/
		unsigned char da[6];            /* Address 1		*/
		unsigned char sa[6];		/* Address 2		*/
		unsigned char bssid[6];		/* BSSID 		*/
		unsigned short seq_ctrl;	/* Sequence Control	*/
                union {
                      struct { 
                              unsigned char time[8];    /* Timestamp	*/
                              unsigned short interval;  /* Interval	*/
                              unsigned short capa;	/* Capabilities	*/
                      } fixed;        
                } beacon;
	} mgmt;
	struct {
                unsigned short duration;		/* Duration/ID  */
                unsigned char ra[6];			/* Address 1	*/
		unsigned char ta[6];			/* Address 2	*/
		unsigned char fcs[4];			/* FCS		*/
        } ctrl_rts;
	struct {
                unsigned short duration;		/* Duration/ID 	*/
                unsigned char ra[6];			/* Address 1	*/
		unsigned char fcs[4];			/* FCS		*/
	} ctrl_cts;
        struct {
		unsigned short duration;		/* Duration/ID 	*/
                unsigned char ra[6];			/* Address 1	*/
                unsigned char fcs[4];			/* FCS		*/
        } ctrl_ack;
        struct {
      		unsigned short aid;			/* AID		*/
		unsigned char bssid[6];			/* BSSID	*/
		unsigned char ta[6];            	/* Address 1	*/
		unsigned char fcs[4];          		/* FCS		*/
        } ctrl_ps_poll;
        struct {
                unsigned short duration;      		/* Duration/ID 	*/
	        unsigned char ra[6];           		/* Address 1	*/
                unsigned char bssid[6];        		/* BSSID	*/
		unsigned char fcs[4];          		/* FCS		*/
        } ctrl_end;
 	struct {
		unsigned short duration;        	/* Duration/ID	*/
                unsigned char ra[6];           		/* Address 1	*/
                unsigned char bssid[6];         	/* BSSID	*/
		unsigned char fcs[4];           	/* FCS		*/
        } ctrl_end_ack;
	struct {
		unsigned short duration;		/* Duration/ID	*/
		unsigned char addr1[6];			/* Address 1	*/
		unsigned char addr2[6];			/* Address 2	*/
		unsigned char addr3[6];			/* Address 3	*/
		unsigned short seq_ctrl;		/* Seq Control	*/
		unsigned char addr4[6];			/* Address 4	*/
        } data;
  } ext;
};

/* 802.11 values -> for Prism header */
struct wvlan_values_prism {
  unsigned int did;
  unsigned short status;
  unsigned short length;
  unsigned int data;
};

/* WellFleet BOFL header */
struct bofl_pkt_header {
  unsigned int pdu; 
  unsigned int seq;
};

/* Prism header, for monitor mode */
struct prism_pkt_header {
  unsigned int code;
  unsigned int length;
  unsigned char device[16];
  struct wvlan_values_prism host_time;
  struct wvlan_values_prism mac_time;
  struct wvlan_values_prism channel;
  struct wvlan_values_prism rssi; 
  struct wvlan_values_prism sq;
  struct wvlan_values_prism signal;
  struct wvlan_values_prism noise;
  struct wvlan_values_prism rate;
  struct wvlan_values_prism istx;
  struct wvlan_values_prism frame_len; 
};

/* TokenRing header */
struct token_pkt_header {
  unsigned char ac;		/* Access control field */
  unsigned char fc;		/* Frame control field */
  unsigned char destination[APSR_TOKEN_ADDR_LEN];
  unsigned char source[APSR_TOKEN_ADDR_LEN];
};

/* FDDI header */
struct fddi_pkt_header {
  unsigned char fc;		/* Frame control field */
  unsigned char destination[APSR_FDDI_ADDR_LEN];
  unsigned char source[APSR_FDDI_ADDR_LEN];
};

/* ARP header */
struct arp_pkt_header {
  unsigned short arp_hat;
  unsigned short arp_pat;
  unsigned char  arp_hal;
  unsigned char  arp_pal;
  unsigned short arp_oc;
};

/* IPv4 header */
struct ipv4_pkt_header {
  unsigned int ipvhl:8;		/* Version + Header length	*/
  unsigned int type_service:8;	/* TOS(Type of Service) field	*/
  unsigned short packet_len;	/* Header+Payload length	*/
  unsigned short ident;		/* Identification field		*/
  unsigned short fragment;	/* Fragment Offset field	*/
  unsigned int time_live:8;	/* TTL(Time to Live) field	*/
  unsigned int protocol:8;	/* Protocol field		*/
  unsigned short sum;		/* Checksum field		*/
  struct in_addr src_ip;        /* Source IP			*/
  struct in_addr dst_ip;     	/* Destination IP 		*/
/*  unsigned int ip_options:24;	# IP options field		*/
/*  unsigned int padding:8;	# Padding field			*/
};


/* Is this really needed? */
/* #if __linux__ || __OSTYPE_LINUX || __OSTYPE_FREEBSD */
struct ipv4_option
{
	  struct  in_addr ipopt_dst;          /* first-hop dst if source routed */
	  char ipopt_list[MAX_IPOPTLEN];      /* options proper */
};
/* #endif */

/* IPv7 header */
struct ipv7_pkt_header {
  unsigned int version:4,	/* Version field		*/
               header_length:12,/* Header Length field		*/
               time_live:16;	/* TTL(Time to Live) field	*/
  unsigned int total_length;	/* Length field			*/
  unsigned char fri[8];		/* Forward route identifier	*/
  unsigned char dst_ip[8];	/* Destination address		*/
  unsigned char src_ip[8];	/* Source address		*/
  unsigned short protocol;	/* Protocol			*/
  unsigned short sum;		/* Checksum			*/
};

/* IPv6 header */
struct ipv6_pkt_header {
  unsigned int vpr:8;	        /* Version + Priority		*/
  unsigned int flow_label:24;	/* Flow Label			*/
  unsigned short payload_len;	/* Payload length		*/
  unsigned int next_header:8;	/* Next header			*/
  unsigned int hop_limit:8;	/* Hop Limit			*/
  struct in6_addr src_ip;	/* Source IP			*/
  struct in6_addr dst_ip;	/* Destination IP		*/
};

/* IPv6 Routing header */
struct ipv6_rt_pkt_header {
  unsigned int next_header:8;   /* Next header			*/
  unsigned int length:8;	/* Length			*/
  unsigned int r_type:8;   	/* Routing Type			*/
  unsigned int seg_left:8;	/* Segments Left		*/
};

/* IPv6 Hop-by-Hop header */
struct ipv6_hh_pkt_header {
  unsigned char next_header;	/* Next header			*/
  unsigned char length;		/* Length 			*/
}; 

/* IP Authentication header */
struct ipah_pkt_header {
  unsigned int next_header:8;	/* Next header			*/
  unsigned int length:8;	/* Length			*/
  unsigned short reserved;	/* Reserved/Not used field	*/
  unsigned int spi;		/* Security Parameter Index	*/
  unsigned int seq_num;		/* Sequence number		*/
};

/* PPPoE header */
struct pppoe_pkt_header {
  unsigned int pppoe_vhl:8;	 /* Version + Header length	*/
  unsigned int pppoe_code:8;	 /* Code field			*/
  unsigned short pppoe_session;	 /* Session ID field		*/
  unsigned short pppoe_length;	 /* Payload length field	*/
};

/* PPPoE Tag header */
struct pppoe_tag_header {
  unsigned short pppoe_tag_type;	 /* Tag type field	*/
  unsigned short pppoe_tag_length;	 /* Tag length field	*/
};

/* PPP header */
struct ppp_pkt_header {
  unsigned short protocol;	 /* Protocol field		*/
};

/* PPP LQR header (rfc 1989) */
struct ppp_lqr_header {
  unsigned int magic_num;
  unsigned int last_out_lqr;
  unsigned int last_out_pkt;
  unsigned int last_out_oct;
  unsigned int peer_in_lqr;
  unsigned int peer_in_pkt;
  unsigned int peer_in_dis;
  unsigned int peer_in_err;
  unsigned int peer_in_oct;
  unsigned int peer_out_lqr;
  unsigned int peer_out_pkt;
  unsigned int peer_out_oct;
  unsigned int save_in_lqr;
  unsigned int save_in_pkt;
  unsigned int save_in_dis;
  unsigned int save_in_err;
  unsigned int save_in_oct;
};

/* PPP LCP header */
struct ppp_lcp_header {
  unsigned char code;		 /* Code field			*/
  unsigned char identifier;	 /* Identifier field		*/
  unsigned short length;	 /* Length field		*/
  union {
      struct {
            unsigned char type;           /* Type field		*/
            unsigned char length;         /* Length field	*/
      } conf; /* PPP LCP Configuration header */
  } ext;
};

/* PPP IPCP header */
struct ppp_ipcp_header {
  unsigned char code;		 /* Code field			*/
  unsigned char identifier;	 /* Identifier field		*/
  unsigned short length;	 /* Length field		*/
  union {
      struct {
	    unsigned char type;           /* Type field		*/
  	    unsigned char length;         /* Length field	*/
      } conf; /* PPP IPCP Configuration header */
  } ext;
};

/* PPP IPV6CP header */
struct ppp_ipv6cp_header {
  unsigned char code;           /* Code field			*/
  unsigned char identifier;     /* Identifier field		*/
  unsigned short length;        /* Length field			*/
  union {
	  struct {
		  unsigned char type;           /* Type field	*/
		  unsigned char length;         /* Length field	*/
	  } conf; /* PPP IPV6CP Configuration header */
  } ext;
};

/* PPP IPXCP header */
struct ppp_ipxcp_header {
  unsigned char code;           /* Code field			*/
  unsigned char identifier;     /* Identifier field		*/
  unsigned short length;        /* Length field			*/
  union {
          struct {
                  unsigned char type;           /* Type field	*/
                  unsigned char length;         /* Length field	*/
          } conf; /* PPP IPXCP Configuration header */
    } ext;
};

/* PPP ATCP header */
struct ppp_atcp_header {
  unsigned char code;           /* Code field			*/
  unsigned char identifier;     /* Identifier field		*/
  unsigned short length;        /* Length field			*/
  union {
        struct {
		unsigned char type;           /* Type field	*/
	        unsigned char length;         /* Length field	*/
		union {
			struct {
				unsigned short at_net;	/* AppleTalk Network  */
				unsigned int at_node;	/* AppleTalk Node     */
			} at_addr;/* AppleTalk Address/Default Router Address */
			struct {
				unsigned short r_proto;	/* Routing Protocol   */
	 		} routing;			/* Routing protocol   */				struct {
				unsigned short c_proto;	/* Compression Protocol */
			} comp;   /* AppleTalk Compression Protocol */
			struct {
				unsigned short s_class;	/* Server-class	*/
				unsigned int s_id;	/* Server-Impl.-ID */
			} server; /* Server Information */
		} types;
        } conf; /* PPP ATCP Configuration header */
  } ext;
};

/* PPP PAP header */
struct ppp_pap_header {
  unsigned char code;		/* Code				*/
  unsigned char identifier;	/* Identifier field		*/
  unsigned short length;	/* Length field			*/
};

/* PPP CHAP header */
struct ppp_chap_header {
  unsigned char code;          /* Code				*/
  unsigned char identifier;    /* Identifier field		*/
  unsigned short length;       /* Length field			*/
};

/* PPP EAP header */
struct ppp_eap_header {
  unsigned char code;          /* Code                          */
  unsigned char identifier;    /* Identifier field              */
  unsigned short length;       /* Length field                  */
};

/* PPP BCP header */
struct ppp_bcp_header {
  unsigned char code;          /* Code                          */
  unsigned char identifier;    /* Identifier field              */
  unsigned short length;       /* Length field                  */
};

/* PPP ECP header */
struct ppp_ecp_header {
  unsigned char code;           /* Code field			*/
  unsigned char identifier;     /* Identifier field		*/
  unsigned short length;        /* Length field			*/
  union {
	  struct {
		  unsigned char type;           /* Type field	*/
		  unsigned char length;         /* Length field	*/
          } conf; /* PPP ECP Configuration header */
  } ext;
};

/* PPP CCP header */
struct ppp_ccp_header {
  unsigned char code;           /* Code field			*/
  unsigned char identifier;     /* Identifier field		*/
  unsigned short length;        /* Length field			*/
  union {
	  struct {
		  unsigned char type;           /* Type field	*/
		  unsigned char length;         /* Length field	*/
          } conf; /* PPP CCP Configuration header */
  } ext;
};

/* PPP Decnet Phase IV CP header */
struct ppp_decnetcp_header {
  unsigned char code;           /* Code field                   */
  unsigned char identifier;     /* Identifier field             */
  unsigned short length;        /* Length field                 */
};

/* PPP OSI Network Layer CP header */
struct ppp_osinlcp_header {
  unsigned char code;           /* Code field                   */
  unsigned char identifier;     /* Identifier field             */
  unsigned short length;        /* Length field                 */
};

/* PPP SNACP header */
struct ppp_snacp_header {
  unsigned char code;          /* Code                          */
  unsigned char identifier;    /* Identifier field              */
  unsigned short length;       /* Length field                  */
};

/* UDP header */
struct udp_pkt_header {
  unsigned short src_port;	/* Source Port			*/
  unsigned short dst_port;	/* Destination Port		*/
  unsigned short length;	/* Length			*/
  unsigned short sum;		/* Checksum			*/
};

/* UDP/TCP over IPv4 pseudo header */
union v4_pseudo_header {
  struct pseudo_header {
	  unsigned int src_ip;
	  unsigned int dst_ip;
	  unsigned char padding;
	  unsigned char protocol;
	  unsigned short length; 
  } ps_hdr;
  unsigned short pa[6];
} v4_ps_hdr;

/* TCP header */
/* This header only applies, if FAVOUR_BSD is set!      */
/* We should check this using some #ifdefs and if it's  */
/* not set, we must use another tcp_pkt_header struct.  */
/*                                                      */
/* arbi: Please don't kill me :)                        */
struct tcp_pkt_header {
  unsigned short src_port;	/* Source Port			*/
  unsigned short dst_port;	/* Destination Port		*/
  unsigned int seq_num;		/* Sequence Number		*/
  unsigned int ack_num;		/* Acknowledgement Number	*/
  unsigned int reserved:4,	/* (unused) */
               data_offset:4,	/* data offset */
               flags:8,		/* Flags			*/
               window:16;	/* Window			*/
  unsigned short sum;		/* Checksum			*/
  unsigned short urgent;	/* Urgent Pointer		*/
};

/* ICMPv4 header */
struct icmpv4_pkt_header {
  unsigned int type:8;	        /* Type				*/
  unsigned int code:8;          /* Code				*/
  unsigned short sum;           /* Checksum			*/
  union {
    struct {
      unsigned short identifier;/* Identifier			*/
      unsigned short seq_num;	/* Sequence Number		*/
    } echo;
    struct {
      unsigned short identifier;/* Identifier			*/
      unsigned short seq_num;   /* Sequence Number		*/
      unsigned int originate;	/* Orginate Timestamp		*/
      unsigned int receive;	/* Receive Timestamp		*/
      unsigned int transmit;	/* Transmit Timestamp		*/
    } timestamp;
    struct {
      unsigned int pointer:8;   /* Pointer			*/
      unsigned int unused:24;	/* Unused			*/
    } para;
    struct in_addr gateway_ip;	/* Gateway    			*/
    struct {
      unsigned short unused;	/* Reserved/Unused		*/
      unsigned short mtu;	/* MTU				*/
    } frag;                     /* Path MTU discovery		*/
    struct {
      unsigned short identifier;/* Identifier			*/
      unsigned short seq_num;   /* Sequence Number		*/
      unsigned int ttl;		/* Time-To-Live			*/
    } dns;
    struct {
        unsigned short unused;  /* Reserved/Unused		*/
        unsigned short pointer; /* Pointer			*/
    } secu; 
  } ext;
};

/* ICMPv6 header */
struct icmpv6_pkt_header {
  unsigned char type;           /* Type				*/
  unsigned char code;           /* Code				*/
  unsigned short sum;           /* Checksum			*/
  union {
    struct {
      unsigned int mtu;        /* MTU				*/
    } packet_too_big;
    struct {
      unsigned int pointer;    /* Pointer			*/
    } parameter_problem;
    struct {
      unsigned short identifier;/* Identifier			*/
      unsigned short seq_num;   /* Sequence Number		*/
    } echo;
    struct {
      unsigned short mrd;	/* Maximum Response Delay	*/
    } group;
  } ext;  
};

/* IGMPv2 header */
struct igmpv2_pkt_header {
  unsigned char type;		/* Type				*/
  unsigned char max_res_time;	/* Max response time		*/
  unsigned short sum;		/* Checksum			*/
  struct in_addr group_addr; 	/* Group address		*/
};

/* RIPv1/RIPv2 header TCP/IP Protocol Suite */
struct rip_pkt_header {
  unsigned int command:8;       /* Command			*/
  unsigned int version:8;  	/* Version			*/
  unsigned short routing_domain;/* zero(v1)/Routing domain(v2)	*/
  unsigned short address_family;/* Address family		*/
  unsigned short route_tag;	/* zero(v1)/Route tag		*/
  struct in_addr ip_address;	/* IP address			*/
  struct in_addr subnet_mask;	/* zero(v1)/Subnet mask(v2)	*/
  struct in_addr next_hop_ip;	/* zero(v1)/Next hop IP(v2)	*/
  unsigned int metric;		/* Metric			*/
};

/* RIPv2 authentication header TCP/IP Protocol Suite */
struct ripv2auth_pkt_header {
  unsigned char command;       	/* Command			*/
  unsigned char version;       	/* Version			*/
  unsigned short unused;	/* Unused			*/
  unsigned short ffff;		/* 0xFFFF			*/
  unsigned short auth_type;	/* Authentication type		*/
  unsigned char auth[16];	/* Authentication data		*/
};

/* RIPng(IPv6) header TCP/IPv6 Protocol Suite */
struct ripng_pkt_header {
  unsigned int command:8;       /* Command			*/
  unsigned int version:8;       /* Version			*/
  unsigned short unused;	/* Unused - zero		*/
};  

/* RIP header IPX Protocol Suite */
struct ipxrip_pkt_header {
  unsigned short type_op;	/* Type Operation field		*/
  unsigned int  network;	/* Network field		*/
  unsigned short hops;		/* Hop field			*/
  unsigned short ticks;		/* Metric of Ticks field	*/
};

/* IPX header */
struct ipx_pkt_header {
  unsigned short sum;		/* Checksum			*/
  unsigned short length;	/* Length(with header)		*/
  unsigned int trans_control:8; /* Transport Control		*/
  unsigned int packet_type:8;	/* Packet type			*/
  unsigned short net_dest[2];	/* destination network		*/
  unsigned char node_dest[6];   /* destination node		*/
  unsigned short socket_dest;   /* destination socket		*/
  unsigned short net_src[2];    /* source network		*/
  unsigned char node_src[6];    /* source node			*/
  unsigned short socket_src;    /* source socket		*/
};

/* SPX header */
struct spx_pkt_header {
  unsigned int con_control:8;   /* Connection Control		*/
  unsigned int datastr_type:8;	/* Datastream Type		*/
  unsigned short source_id;	/* Source connection ID		*/
  unsigned short dest_id;	/* Destination connection ID	*/
  unsigned short seq_num; 	/* Sequence number		*/
  unsigned short ack_num;	/* Acknowledge number		*/
  unsigned short alloc_num;	/* Allocation number		*/
  union {
    struct {
      unsigned short negotiate_size; /* Negotiate Size		*/
    } spx2;
  } ext;
};

/* OSPFv2 header */
struct ospfv2_pkt_header {
  unsigned int version:8;	/* Version			*/
  unsigned int type:8;		/* Type				*/
  unsigned short length;	/* Length			*/
  struct in_addr router_id;	/* Router ID			*/
  struct in_addr area_id;	/* Area ID			*/
  unsigned short sum;		/* Checksum			*/
  unsigned short autype;	/* Authentication Type		*/
  unsigned char authentication[8]; /* Authentication 		*/
};

/* OSPFv3 header */
struct ospfv3_pkt_header {
  unsigned int version:8;       /* Version			*/
  unsigned int type:8;          /* Type				*/
  unsigned short length;        /* Length			*/
  struct in_addr router_id;     /* Router ID			*/
  struct in_addr area_id;       /* Area ID			*/
  unsigned short sum;           /* Checksum			*/
  unsigned int instance_id:8;	/* Instance ID			*/
  unsigned int reserved:8;	/* Reserved			*/
};

/* OSPF Hello packet header */
struct ospf_hello_pkt_header {
  struct in_addr net_mask;	/* Network Mask			*/
  unsigned short hello_int;	/* Hello Interval		*/
  unsigned int options:8;	/* Options			*/
  unsigned int rtr_pri:8;	/* Router Priority		*/
  unsigned int rtr_dead_int;	/* Route Dead Intervall		*/
  unsigned int designated_rtr;	/* Designated Router		*/
  unsigned int backup_des_rtr;	/* Backup Designated Router	*/
  unsigned int neighbor;	/* Neighbor  			*/
};

/* OSPF Database Description packet header */
struct ospf_database_desc_pkt_header {
  unsigned int reserved1:8;	/* Reserved			*/
  unsigned int reserved2:8;	/* Reserved			*/
  unsigned int options:8;       /* Options			*/
  unsigned int i_m_ms:8;	/* I-bit & M-bit & MS-bit	*/
  unsigned int dd_seq_num;	/* DD Sequence Number		*/
};

/* OSPF Linke State Request packet header */
struct ospf_ls_request_pkt_header {
  unsigned int ls_type;		/* Link State type		*/
  unsigned int ls_id;		/* Link State ID		*/
  unsigned int adv_rtr;		/* Advertising Router		*/
};

/* OSPF Link State Advertisement packet header */
struct ospf_ls_adv_pkt_header {
  unsigned short ls_age;	/* Link State Age		*/
  unsigned int options:8;	/* Options			*/
  unsigned int ls_type;		/* Link State type		*/
  unsigned int ls_id;		/* Link State ID		*/
  unsigned int adv_rtr;         /* Advertising Router		*/
  unsigned int ls_seq_num;      /* Link State Sequence Number	*/
  unsigned short sum;           /* Checksum			*/
  unsigned short length;        /* Length			*/
};

/* EGP packet header */
struct egp_pkt_header {
  unsigned int version:8;	/* Version			*/
  unsigned int type:8;		/* Type				*/
  unsigned int code:8;		/* Code				*/
  unsigned int status:8;	/* Status			*/
  unsigned short sum;		/* Checksum			*/
  unsigned short auto_sys;	/* Autonomus System		*/
  unsigned short sequence;	/* Sequence			*/
  union {
    struct {
      unsigned short hello_int; /* Hello Interval 		*/
      unsigned short poll_int;  /* Poll Interval 		*/
    } nam;			/* Neighbor Acquisition Message	*/
    struct {
      unsigned short reason;    /* Reason			*/
    } error;
    struct {
      unsigned short reserved;  /* Reserved			*/
      struct in_addr source;    /* IP Souce Network		*/
    } poll;
  } ext;
};

/* BGP packet header */
struct bgp_pkt_header {
  unsigned char marker[16];	/* Marker			*/
  unsigned short length;	/* Length			*/
  unsigned char type;		/* Type				*/
};

/* IPComp packet header */
struct ipcomp_pkt_header {
  unsigned int next_header:8;	/* Next header			*/
  unsigned int flags:8;		/* Flags			*/
  unsigned short cpi;		/* Compression Parameter Index	*/
};

/* NARP(NBMA Address Resolution Protocol) packet header */
struct narp_pkt_header {
  unsigned int version:8;	/* Version			*/
  unsigned int hop:8;		/* Hop Count			*/
  unsigned short sum;		/* Checksum			*/
  unsigned int type:8;		/* Type				*/
  unsigned int code:8;		/* Code				*/
  unsigned short reserved;	/* 16 bits reserved		*/
  struct in_addr dst_ip;	/* Destination IP		*/
  struct in_addr src_ip;	/* Source IP			*/
  unsigned int length:8;	/* Length			*/
};

/* GGP packet header */
struct ggp_pkt_header {
  unsigned int gtype:8;		/* Gateway type			*/
  unsigned int unused:8;	/* Unused			*/
  unsigned short seq_num;	/* Sequence number		*/
};

/* IGRP packet header */
struct igrp_pkt_header {
  unsigned int code_version:8;	/* Opcode/Version		*/
  unsigned int edition:8;  	/* Edition			*/
  unsigned short autosys;	/* Autosys			*/
  unsigned short interior;	/* Interior			*/
  unsigned short system;	/* System			*/
  unsigned short exterior;	/* Exterior			*/
  unsigned short sum;		/* Checksum			*/
};

/* EIGRP packet header */
struct eigrp_pkt_header {
  unsigned char version;        /* Version			*/
  unsigned char opcode;		/* Opcode			*/
  unsigned short sum;		/* Checksum			*/
  unsigned int flags;		/* Flags			*/
  unsigned int seq_num;		/* Sequence number		*/
  unsigned int ack_num;		/* Acknowledge sequence number	*/
  unsigned int autosys;		/* Autonmous System		*/
};	

/* VRRP packet header */
struct vrrp_pkt_header {
  unsigned char version_type;	/* Version/Type			*/
  unsigned char vrid;         	/* Virtual Rtr ID		*/
  unsigned char priority;	/* Priority			*/
  unsigned char count_ip;	/* Count IP Addresses		*/
  unsigned char auth_type;	/* Auth type			*/
  unsigned char adver_int;	/* Advertisement Interval	*/
  unsigned short sum;		/* Checksum			*/
};

/* SCTP packet header */
struct sctp_pkt_header {
  unsigned short src_port;	/* Source Port			*/
  unsigned short dst_port;	/* Destination Port		*/
  unsigned int v_tag;		/* Verification Tag		*/
  unsigned int sum;		/* Checksum			*/
};

/* PIM packet header */
struct pim_pkt_header {
  unsigned int ver_type:8;	/* Version(4 bit) / Type(4 bit)	*/
  unsigned int reserved:8;	/* Reserved			*/
  unsigned short sum;		/* Checksum 			*/
};

/* IP ESP header */
struct ipesp_pkt_header {
  unsigned int spi;		/* SPI				*/
  unsigned int seq_num;		/* Sequence Number		*/
};

/* SLL header */
struct sll_pkt_header {
  unsigned short sll_pkt_type;	/* Packet type			*/
  unsigned short sll_lla_type;	/* Link-Layer Type		*/
  unsigned short sll_lla_len;	/* Link-Layer Length		*/
  unsigned char sll_addr[8];	/* Link-Layer Address		*/
  unsigned short sll_protocol;	/* Protocol			*/
};

/* SunATM header */
struct sunatm_pkt_header {
  unsigned char direction; 	/* Direction 			*/
  unsigned char vpi;		/* VPI				*/
  unsigned short vci;		/* VCI 				*/
};

/* GRE header */
struct gre_pkt_header {
  unsigned short flags;		/* Flags			*/
  unsigned short protocol;	/* Protocol 			*/
};

/* IFMP header */
struct ifmp_pkt_header {
  unsigned char version;	/* Version			*/
  unsigned char opcode;		/* Opcode			*/
  unsigned short sum;		/* Checksum			*/
  union {
	  struct {
		  unsigned int sender_inst;	/* Sender Instance	*/
		  unsigned int peer_inst;	/* Peer Instance	*/
		  unsigned int peer_ident;	/* Peer Identity	*/
		  unsigned int peer_next_seq;	/* Peer Next seq number	*/
		  unsigned short res1;		/* Reserved(1)		*/
		  unsigned char res2;		/* Reserved(2)		*/
		  unsigned char max_ack_intvl; 	/* Max Ack Intvl	*/
	  } adjacency;
	  struct {
		  unsigned int sender_inst;     /* Sender Instance	*/
		  unsigned int peer_inst;       /* Peer Instance	*/
		  unsigned int seq_num;		/* Sequence Number	*/
	  } redirect;
  } ext;
};

/* CDP header */
struct cdp_pkt_header {
  unsigned int version:8;	/* Version			*/
  unsigned int ttl:8;		/* Time-to-Live			*/
  unsigned short sum;		/* Checksum			*/
};

/* SLIP header */
struct slip_pkt_header {
  unsigned char first_end;
  struct ipv4_pkt_hdr *ipv4_header;
  union {
	  struct ipv4_options *ipv4_opt;
  } ipv4_options;
  unsigned char last_end;
};

/* IRTP header */
struct irtp_pkt_header {
  unsigned char type;    	/* Type				*/
  unsigned char port;		/* Port				*/
  unsigned short seq_num;       /* Sequence number		*/
  unsigned short length;	/* Length			*/
  unsigned short sum;          	/* Checksum			*/
};

/* MOPv3(DEC Request Program header */
struct mopv3_dumpload_rqprg_pkt_header {
  unsigned char code;
  unsigned char dev_type;
  unsigned char format_version;
  unsigned char prog_type;
};

/* MOPv3(DEC) Request Memory Load header */
struct mopv3_dumpload_rqmemld_pkt_header {
  unsigned char code;
  unsigned char load_num;
  unsigned char error;
};

/* MOPv3(DEC) Memory Load header */
struct mopv3_dumpload_memld_pkt_header {
  unsigned char code;
  unsigned char load_num;
  unsigned char load_addr[4];
};

/* MOPv3(DEC) Parameter Load with Transfer Address header */
struct mopv3_dumpload_parldta_pkt_header {
  unsigned char code;
  unsigned char load_num;
  unsigned char para;
  unsigned char trans_addr[4];
};

/* CIP header */
struct cip_pkt_header {
  unsigned short int cpf;
  unsigned short int type;
  unsigned short int len_address_data;
  unsigned int identifier;
  unsigned int sequence;
  unsigned short int data_type;
  unsigned short int len;
  char *data;
};

/* MPLS header */
struct mpls_pkt_header {
  /* Label(20), Experimental(3), Bottom of Stack(1),TTL(8)*/
  unsigned int header;
};

/* GSMP header */
struct gsmp_pkt_header {
  unsigned char version;
  unsigned char type;
  unsigned char result;
  unsigned char code;
};

/* CBT header */
struct cbt_pkt_header {
  unsigned char version_type;	/* Version + Type		*/
  unsigned char addr_length;	/* Address Length		*/
  unsigned short sum;		/* Checksum			*/
};

/* RSVP header */
struct rsvp_pkt_header {
  unsigned char version_flags;	/* 4 Bit Versions & 4 Bit Flags */
  unsigned char msg_type;	/* Message Type */
  unsigned short sum;		/* Checksum */
  unsigned char send_ttl;	/* Send TTL */
  unsigned char reserved;	/* Reserved */
  unsigned short length;	/* Length */
};

/* Cisco HDLC header */
struct chdlc_pkt_header {
  unsigned char address;        /* Address */
  unsigned char unknown;	/* Don't know what this field says (MP) */
  unsigned short protocol;      /* Protocol */
};

/* MSDP header */
struct msdp_pkt_header {
  unsigned char type;		/* Type */
  unsigned short length;	/* Length */
};

/* CGMP header */
struct cgmp_pkt_header {
  unsigned int ver_type:8,
               reserved:16,
	       count:8;
  unsigned char gda[6];
  unsigned char usa[6];
};

/* VTP header */
struct vtp_pkt_header {
  unsigned char version;
  unsigned char code;
};

/* SDRP header */
struct sdrp_pkt_header {
  unsigned int version:3,
               data_ctrl_flag:1,
	       loose_strict_route:1,
	       probe_indicator:1,
	       reserved:2,
	       hop_count:8,
	       source_route_protocol:8,
	       payload_protocol_type:8;
  unsigned int source_route_identifier;
  struct in_addr target_router;
  unsigned int prefix;
  unsigned char prefix_length;
  unsigned char notification;
  unsigned char source_route_length;
  unsigned char next_hop_ptr;
};

/* EtherIP header */
struct etherip_pkt_header {
  unsigned int version:4,
  	       reserved:12,
	       temp_dontuse:16;
};

/* NETBLK header */
struct netblk_pkt_header {
  unsigned short checksum;
  unsigned char version;
  unsigned char type;
  unsigned short length;
  unsigned short local_port;
  unsigned short forein_port;
  unsigned short lap;		/* Longword Alignment Padding */
};

/* TMux header */
struct tmux_mini_pkt_header {
  unsigned short length;
  unsigned char protocol_id;
  unsigned char checksum;
};

/* RDPv1 header */
struct rdpv1_pkt_header {
  unsigned char flags;
  unsigned char header_length;
  unsigned char source_port;
  unsigned char destination_port;
  unsigned short data_length;
  unsigned int seq_num;
  unsigned int ack_num;
  unsigned int sum;
};

/* RDPv2 header */
struct rdpv2_pkt_header {
  unsigned char flags;
  unsigned char header_length;
  unsigned short source_port;
  unsigned short destination_port;
  unsigned short data_length;
  unsigned int seq_num;
  unsigned int ack_num;
  unsigned short sum;
};

/* MTP header */
struct mtp_pkt_header {
  unsigned short dst_port;
  unsigned short src_port;
  unsigned short length;
  unsigned short sum;
  unsigned char version;
  unsigned char type;
  unsigned char type_modifier;
  unsigned char sub_channel;
  unsigned int src_connection_identifier;
  unsigned int dst_connection_identifier;
};
  
/* PGM header */
struct pgm_pkt_header {
  unsigned short source_port;
  unsigned short dest_port;
  unsigned char pgm_type;
  unsigned char options;
  unsigned short sum;
  unsigned char gsid[6];
  unsigned short tsdu_length;
};

#if __OSTYPE_LINUX && HAVE_LIBBLUETOOTH
struct frame {
	void    *data;
	int     data_len;
	void    *ptr;
	int     len;
	int     in;
	int     handle;
	long    flags;
	struct  timeval ts;
};
#endif

