/* IPv6 definitions */
#define IP6_NEXTHDR_HOP         0       /* Hop-by-Hop option header	*/
#define IP6_NEXTHDR_TCP         6       /* TCP packet			*/
#define IP6_NEXTHDR_EGP         8       /* EGP packet			*/
#define IP6_NEXTHDR_UDP         17      /* UDP packet			*/
#define IP6_NEXTHDR_IP6         41      /* IPv6 in IPv6			*/
#define IP6_NEXTHDR_ROUTING     43      /* Routing header		*/
#define IP6_NEXTHDR_FRAGMENT    44      /* Fragmentation/reassembly hdr	*/
#define IP6_NEXTHDR_ESP         50      /* Encapsulating security payl.	*/
#define IP6_NEXTHDR_AUTH        51      /* Authentication header	*/
#define IP6_NEXTHDR_ICMP        58      /* ICMP				*/
#define IP6_NEXTHDR_NONE        59      /* No next header		*/
#define IP6_NEXTHDR_DEST        60      /* Destination options header	*/
#define IP6_NEXTHDR_PIM		103	/* PIM				*/
#define IP6_NEXTHDR_COMP        108     /* IP Payl. Comp. Protocol	*/


/* IPv6 macros */
/* This will give a warning under AIX, but it'll hopefully compile. */
#if !defined(IPV6_VERSION) || defined(__OSTYPE_AIX)
#define IPV6_VERSION(ipv6_ps_header) (((ipv6_ps_header)->vpr & 0xf0) >> 4)
#endif

#if !defined(IPV6_PRIORITY) || defined(__OSTYPE_AIX)
#define IPV6_PRIORITY(ipv6_ps_header) ((ipv6_ps_header)->vpr & 0x0f)
#endif

#ifndef IN6_IS_ADDR_UNSPECIFIED
#define IN6_IS_ADDR_UNSPECIFIED(a) \
	(((unsigned int *) (a))[0] == 0 && ((unsigned int *) (a))[1] == 0 && \
	((unsigned int *) (a))[2] == 0 && ((unsigned int *) (a))[3] == 0)
#endif

#ifndef IN6_IS_ADDR_LOOPBACK
#define IN6_IS_ADDR_LOOPBACK(a) \
	(((unsigned int *) (a))[0] == 0 && ((unsigned int *) (a))[1] == 0 && \
	((unsigned int *) (a))[2] == 0 && ((unsigned int *) (a))[3] == apsr_htonl (1))
#endif

#ifndef IN6_IS_ADDR_MULTICAST
#define IN6_IS_ADDR_MULTICAST(a) (((unsigned char *) (a))[0] == 0xff)
#endif

#ifndef IN6_IS_ADDR_LINKLOCAL
#define IN6_IS_ADDR_LINKLOCAL(a) \
	((((unsigned int *) (a))[0] & apsr_htonl (0xffc00000)) == apsr_htonl (0xfe800000))
#endif

#ifndef IN6_IS_ADDR_SITELOCAL
#define IN6_IS_ADDR_SITELOCAL(a) \
	((((unsigned int *) (a))[0] & apsr_htonl (0xffc00000)) == apsr_htonl (0xfec00000))
#endif

#ifndef IN6_IS_ADDR_V4MAPPED
#define IN6_IS_ADDR_V4MAPPED(a) \
	((((unsigned int *) (a))[0] == 0) && (((unsigned int *) (a))[1] == 0) && \
	(((unsigned int *) (a))[2] == apsr_htonl (0xffff)))
#endif

#ifndef IN6_IS_ADDR_V4COMPAT
#define IN6_IS_ADDR_V4COMPAT(a) \
	((((unsigned int *) (a))[0] == 0) && (((unsigned int *) (a))[1] == 0) && \
	(((unsigned int *) (a))[2] == 0) && (apsr_ntohl (((unsigned int *) (a))[3]) > 1))
#endif


#ifndef IN6_ARE_ADDR_EQUAL
#define IN6_ARE_ADDR_EQUAL(a,b) \
	((((unsigned int *) (a))[0] == ((unsigned int *) (b))[0]) && \
	(((unsigned int *) (a))[1] == ((unsigned int *) (b))[1]) && \
	(((unsigned int *) (a))[2] == ((unsigned int *) (b))[2]) && \
	(((unsigned int *) (a))[3] == ((unsigned int *) (b))[3]))
#endif

