#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="lo";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header, 20 bytes */
	/* for the IPv4 header, 20 bytes for the TCP header */
	/* and 18 bytes for the CIP header (and x bytes for the data field) */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv4_H + APSR_TCP_H + APSR_CIP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

 	my_ret = apsr_ipv4(4,			// IP version
			   5,			// IP header length
			   20,			// IP length
			   0x1111,		// IP Type of Service (ToS)
			   45360,		// IP Identification (ID)
			   0x4000,		// IP Fragementation offset
			   64,			// IP Time to live (TTL)
			   APSR_PROTO_TCP,	// IP transport layer protocol
			   inet_addr("127.0.0.1"), // Source IP
			   inet_addr("127.0.0.2"), // Destination IP
			   0,			// IP checksum
			   NULL,		// IP options
			   0,			// IP option size
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv4 failed!");

	my_ret = apsr_tcp(44552,		// TCP source port
			  44552,		// TCP destination port
			  2097900844,		// TCP sequence number
			  0x12345678,		// TCP ack number
			  APSR_TCP_SYN,		// TCP flags
			  5,			// TCP data offset
			  0,			// TCP reserved flag 1
			  0,			// TCP reserved flag 2
			  30660,		// TCP window size
			  34281,		// TCP urgent pointer
			  0,			// TCP checksum
			  NULL,			// TCP payload
			  0,			// TCP payload length
			  buf + APSR_ETH_H
			      + APSR_IPv4_H);	// Packet buffer
	
	if(my_ret == -1) ERROR("apsr_tcp failed!");

	my_ret = apsr_cip(2,			// CIP common packet format
			  2,			// CIP type
			  6,			// CIP len address data
			  0xAAFF,		// CIP identifier
			  0xFFFF,		// CIP sequence number
			  8,			// CIP data type
			  0,			// CIP length
			  NULL,			// CIP data
			  NULL,			// CIP payload
			  0,			// CIP payload length
			  buf + APSR_ETH_H
			      + APSR_IPv4_H
			      + APSR_TCP_H);	// Packet buffer

	/* Calculate the checksum */
	
//	apsr_checksum(buf+APSR_ETH_H, APSR_PROTO_TCP, APSR_TCP_H+APSR_IPv4_H+APSR_ETH_H);

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_IPv4_H + APSR_TCP_H + APSR_CIP_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

