#include <apsrlib/apsrlib.h>

#define MODULE_VERSION          "0.02"
#define MODULE_NAME             "dcap"
#define MODULE_DESCRIPTION      "Data Link Switching Client Access Protocol (DCAP) sniffer"
#define MODULE_IDENT_FAMILY     AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	1973
#define MODULE_IDENT_ID2        1973

#define MODULE_FILTER		"tcp and port 1973"

#define DCAP_PROTOCOL(dcap_ps_header) (((dcap_ps_header)->proto_version & 0xf0) >> 4)
#define DCAP_VERSION(dcap_ps_header) ((dcap_ps_header)->proto_version & 0x0f)

static const char *mod_dcap_type[]= {
	" (Unknown)",
	" (CAN_U_REACH)",
	" (I_CAN_REACH)",
	" (I_CANNOT_REACH)",
	" (START_DL)",
	" (DL_STARTED)",
	" (START_DL_FAILED)",
	" (XID_FRAME)",
	" (CONTACT_STN)",
	" (STN_CONTACTED)",
	" (DATA_FRAME)",
	" (INFO_FRAME)",
	" (HALT_DL)",
	" (HALT_DL_NOACK)",
	" (DL_HALTED)",
	" (FCM_FRAME)",
	" (DGRM_FRAME)",
	" (CAP_XCHANGE)",
	" (CLOSE_PEER_REQUEST)",
	" (CLOSE_PEER_RESPONSE)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
	" (PEER_TEST_REQ)",
	" (PEER_TEST_RSP)",
};

struct mod_dcap_header {
	unsigned char 	proto_version;
	unsigned char 	type;
	unsigned short 	length;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version     = MODULE_VERSION;
	module_info.name        = MODULE_NAME;
	module_info.desc        = MODULE_DESCRIPTION;
	module_info.bpf_filter  = MODULE_FILTER;
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,                 unsigned short id1, unsigned short id2)
{
  struct mod_dcap_header *dcap_header;
	
  if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_PN1 == protocol_number && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	dcap_header = (struct mod_dcap_header *) buf;

	apsr_output("- DCAP header -\n  Protocol: %u", DCAP_PROTOCOL(dcap_header));
	apsr_output(", Version: %u\n", DCAP_VERSION(dcap_header));
	apsr_output("  Type: %u", dcap_header->type);
	if(verbose)
		apsr_output("%s", mod_dcap_type[dcap_header->type]);
	apsr_output(", Length: %u\n", apsr_ntohs(dcap_header->length));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		     family, protocol_number, id1, id2);
  #endif

  return(RET_SUCCESS);
}

