#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"gnutella"
#define MODULE_DESCRIPTION	"Example gnutella protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	6346	
#define MODULE_IDENT_ID2	6346

#define MODULE_FILTER		"tcp and port 6346"

#define APSR_GNUTELLA_DESCRIPTOR_PING	      0x00
#define APSR_GNUTELLA_DESCRIPTOR_PONG         0x01
#define APSR_GNUTELLA_DESCRIPTOR_END	      0x02
#define APSR_GNUTELLA_DESCRIPTOR_QRP	      0x30
#define APSR_GNUTELLA_DESCRIPTOR_VENDOR	      0x31
#define APSR_GNUTELLA_DESCRIPTOR_STD_VENDOR   0x32
#define APSR_GNUTELLA_DESCRIPTOR_PUSH         0x40
#define APSR_GNUTELLA_DESCRIPTOR_QUERY        0x80
#define APSR_GNUTELLA_DESCRIPTOR_QUERYHIT     0x81

/* TODO: Query variable field				*/
/* TODO: QueryHit variable field			*/
/* TODO: multiple GNUTELLA header in one tcp packet	*/

#define APSR_GNUTELLA_HEADER_SIZE	23

struct mod_gnutella_header {
	unsigned char descriptor_id[16];
	unsigned char payload_descriptor;
	unsigned char ttl;
	unsigned char hops;
	unsigned char payload_length[4];
};

struct mod_gnutella_pong_header {
	unsigned short port;
	unsigned char ip[4];
	unsigned char files_shared[4];
	unsigned char kilo_shared[4];
};

struct mod_gnutella_queryhit_header {
	unsigned char number_hits;
	unsigned short port;
	unsigned char ip[4];
	unsigned char speed[4];
};

struct mod_gnutella_query_header {
	unsigned short minimum_speed;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_gnutella_header *gnutella_header;
  struct mod_gnutella_pong_header *pong_header;
  struct mod_gnutella_query_header *query_header;
  struct mod_gnutella_queryhit_header *queryhit_header;
  unsigned short i=0;

  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    gnutella_header = (struct mod_gnutella_header *)buf;
	    apsr_output("- Gnutella(v0.4) Header -\n");
	    if(memcmp(buf, "GNUTELLA CONNECT", 16) == 0) 
		    apsr_output("  - Gnutella Connect string -\n");
	    else if(memcmp(buf, "GNUTELLA OK", 9) == 0) 
		    apsr_output("  - Gnutella Connect Ok string -\n");
	    else if(memcmp(buf, "GNUTELLA", 8) == 0) 
		    apsr_output("  - Gnuttella unknown HTTP request/reply -\n");
	    else {
  	      apsr_output("  Descriptor ID: ");
	      for(i = 0; i <= 15; i++) {
		    apsr_output("%02X", 
				    gnutella_header->descriptor_id[i]);
	      }
              apsr_output("\n  Payload Descriptor: 0x%02x", 
			    gnutella_header->payload_descriptor);
	      if(verbose) {
		    switch(gnutella_header->payload_descriptor) {
			    case APSR_GNUTELLA_DESCRIPTOR_PING:
			      apsr_output(" (Ping)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_PONG:
			      apsr_output(" (Pong)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_PUSH:
			      apsr_output(" (Push)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_QUERY:
			      apsr_output(" (Query)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_QUERYHIT:
			      apsr_output(" (QueryHit)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_QRP:
			      apsr_output(" (QRP)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_END:
			      apsr_output(" (End)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_VENDOR:
			      apsr_output(" (Vendor)");
			      break;
			    case APSR_GNUTELLA_DESCRIPTOR_STD_VENDOR:
			      apsr_output(" (Std. Vendor)");
			      break;
			    default:
			      apsr_output(" (unknown)");
			      break;
		    }
              }		    
	      apsr_output("\n  TTL: %u, Hops: %u\n",     
			    gnutella_header->ttl,
			    gnutella_header->hops);
	      apsr_output("  Payload Length: %u\n",
			    gnutella_header->payload_length[0]);
	      if(gnutella_header->payload_descriptor == APSR_GNUTELLA_DESCRIPTOR_PONG) {
		    pong_header = (struct mod_gnutella_pong_header *)(buf+APSR_GNUTELLA_HEADER_SIZE);
		    apsr_output("  Port: %u, IP: %u.%u.%u.%u\n",
			pong_header->port, pong_header->ip[0], 
			pong_header->ip[1], pong_header->ip[2], 
			pong_header->ip[3]);
		    apsr_output("  Number of Files: %u, KiloBytes shared: %u\n",
			pong_header->files_shared[0],
			pong_header->kilo_shared[0]);
	      }		
	      else if(gnutella_header->payload_descriptor == APSR_GNUTELLA_DESCRIPTOR_QUERY) {
		    query_header = (struct mod_gnutella_query_header *)(buf+APSR_GNUTELLA_HEADER_SIZE);
		    apsr_output("  Minimum Speed: %u kb/s\n",
		        query_header->minimum_speed);
	      }
	      else if(gnutella_header->payload_descriptor == APSR_GNUTELLA_DESCRIPTOR_QUERYHIT) {
		    queryhit_header = (struct mod_gnutella_queryhit_header *)(buf+APSR_GNUTELLA_HEADER_SIZE);  
		    apsr_output("  Number of Hits: %u, Port: %u\n",
		       	queryhit_header->number_hits,
			queryhit_header->port);
		    apsr_output("  IP: %u.%u.%u.%u, Speed: %u kb/s\n",
			queryhit_header->ip[0], queryhit_header->ip[1], 
			queryhit_header->ip[2], queryhit_header->ip[3], 
			queryhit_header->speed[0]);
	      }
     	    }	    
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
