#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"socks"
#define MODULE_DESCRIPTION	"Example Socks protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	1080	
#define MODULE_IDENT_ID2	1080

#define MODULE_FILTER		"tcp and port 1080"

static const char *mod_socks_methods[]= {
	" (No Authentication)",
	" (GSSAPI)",
	" (Username/Password)",
	" (Challenge-Handshake Authentication Protocol)",
	" (Reserved)",
	" (Challenge-Response Authentication Method)",
	" (Secure Sockets Layer)",
	" (NDS Authentication)",
	" (Multi-Authentication Framework)",
};

struct mod_socks_header {
	unsigned char version;
	unsigned char method;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_socks_header *socks_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    socks_header = (struct mod_socks_header *)buf;
	    apsr_output("- SOCKS Header -\n");
	    apsr_output("  Version: %u, Method: %u", socks_header->version,
			    socks_header->method);
	    if(verbose) 
		    apsr_output("%s", mod_socks_methods[socks_header->method]);
	    apsr_output("\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
